# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

from shrike import compliant_logging
from shrike.compliant_logging.constants import DataCategory
from shrike.compliant_logging.logging import (
    CompliantLogger,
    get_aml_context,
)
from shrike.compliant_logging.exceptions import PublicRuntimeError
from shrike.compliant_logging import is_eyesoff
from pathlib import Path
import logging
import pytest
import re
import sys
import vaex
import pandas as pd
import numpy as np
from pyspark.sql import SparkSession
from pyspark.sql.types import (
    StructType,
    StructField,
    FloatType,
    IntegerType,
    StringType,
)
from unittest.mock import patch

from shrike._core import stream_handler


def test_basic_config():
    logging.warning("before basic config")

    logging.basicConfig()
    logging.warning("warning from test_basic_config")

    log = logging.getLogger("foo")
    log.warning("warning from foo logger")


@pytest.mark.parametrize("level", ["debug", "info", "warning", "error", "critical"])
def test_data_category_and_log_info_works_as_expected(level):
    compliant_logging.enable_compliant_logging()

    log = logging.getLogger()
    log.setLevel(level.upper())

    assert isinstance(log, compliant_logging.logging.CompliantLogger)

    with stream_handler(
        log, format="%(prefix)s%(levelname)s:%(name)s:%(message)s"
    ) as context:
        func = getattr(log, level)
        func("PRIVATE")
        func("public", category=DataCategory.PUBLIC)
        logs = str(context)

    assert re.search(r"^SystemLog\:.*public$", logs, flags=re.MULTILINE)
    assert not re.search(r"^SystemLog\:.*\:PRIVATE", logs, flags=re.MULTILINE)


def test_non_category_aware_logging_works_as_expected():
    compliant_logging.enable_compliant_logging()

    log = logging.getLogger()
    extra = {"test_name": "", "test_id": ""}
    assert isinstance(log, compliant_logging.logging.CompliantLogger)
    with stream_handler(
        log, "%(test_name)s:%(test_id)s %(prefix)s%(levelname)s:%(name)s:%(message)s"
    ) as context:
        log.log(1, "message", extra={"test_name": "Test", "test_id": 1})
        log.debug("message", extra={"test_name": "Test2", "test_id": 0})
        log.info("message", extra=extra)
        log.warning("message", extra={"test_name": "My", "test_id": "a"})
        try:
            1 / 0
        except Exception as e:
            logging.error(
                "Error at division",
                exc_info=e,
                stack_info=True,
                extra={"test_name": "Test", "test_id": 1},
            )
        log.critical("message", extra=extra, stack_info=True)
        logs = str(context)

    assert re.search(r"^Test:1 Level 1:root:message$", logs, flags=re.MULTILINE)
    assert re.search(r"^Test2:0 DEBUG:root:message$", logs, flags=re.MULTILINE)
    assert re.search(r"^: INFO:root:message$", logs, flags=re.MULTILINE)
    assert re.search(r"^My:a WARNING:root:message$", logs, flags=re.MULTILINE)
    assert re.search(
        r"^Test:1 ERROR:root:Error at division\nTraceback(.*\n){4}Stack",
        logs,
        flags=re.MULTILINE,
    )
    assert re.search(r"^: CRITICAL:root:message\nStack", logs, flags=re.MULTILINE)


@pytest.mark.parametrize("exec_type,message", [(ArithmeticError, "1+1 != 3")])
def test_exception_works_as_expected(exec_type, message):
    compliant_logging.enable_compliant_logging()
    log = logging.getLogger()
    assert isinstance(log, compliant_logging.logging.CompliantLogger)

    with stream_handler(log, "%(prefix)s%(levelname)s:%(name)s:%(message)s") as context:
        try:
            raise exec_type(message)
        except exec_type:
            log.error("foo", category=DataCategory.PUBLIC)
        logs = str(context)

    assert re.search(r"^SystemLog\:.*foo$", logs, flags=re.MULTILINE)


def test_all_the_stuff():
    compliant_logging.enable_compliant_logging()
    log = logging.getLogger("foo")
    log.info("public", category=DataCategory.PUBLIC)
    log.info("PRIVATE", category=DataCategory.PRIVATE)

    log.info("PRIVATE2")


@pytest.mark.skipif(sys.version_info < (3, 8), reason="Requires Python >= 3.8")
def test_enable_compliant_logging_sets_force():
    # Pytest adds handlers to the root logger by default.
    initial_handlers = list(logging.root.handlers)

    compliant_logging.enable_compliant_logging()

    assert len(logging.root.handlers) == 1
    assert all(h not in logging.root.handlers for h in initial_handlers)


def test_warn_if_root_handlers_already_exist(capsys):
    # Pytest adds handlers to the root logger by default.

    compliant_logging.enable_compliant_logging()

    # https://docs.pytest.org/en/stable/capture.html
    stderr = capsys.readouterr().err
    assert "SystemLog:The root logger already has handlers set!" in stderr


def test_deprecated_enable_confidential_logging(capsys):
    """Pytest the pending deprecation of enable_confidential_logging"""

    compliant_logging.enable_confidential_logging()

    # https://docs.pytest.org/en/stable/capture.html
    stderr = capsys.readouterr().err
    assert (
        "SystemLog: The function enable_confidential_logging() is on the way "
        "to deprecation. Please use enable_compliant_logging() instead." in stderr
    )


def test_get_aml_context():
    """Pytest CompliantLogger._get_aml_context"""
    assert (
        compliant_logging.logging.CompliantLogger(
            name="test_get_aml_context"
        )._get_aml_context()
        is get_aml_context()
    )


def test_logging_aml_metric_single_value():
    """Pytest CompliantLogger.metric_value"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()
    log.metric_value(name="test_log_value", value=0, category=DataCategory.PUBLIC)
    with stream_handler(log, "") as context:
        log.metric_value(name="test_log_value", value=0, category=DataCategory.PRIVATE)
        logs = str(context)
    assert "NumbericMetric  | test_log_value:None | 0" in logs


def test_logging_aml_metric_image():
    """Pytest CompliantLogger.metric_image"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()
    log.metric_image(
        name="dummy_image",
        path=str(Path(__file__).parent.resolve() / "data/dummy_image.png"),
        category=DataCategory.PUBLIC,
    )

    log.metric_image(
        path=str(Path(__file__).parent.resolve() / "data/dummy_image.png"),
        category=DataCategory.PUBLIC,
    )

    with stream_handler(log, "") as context:
        log.metric_image(
            name="test_log_image",
            path=str(Path(__file__).parent.resolve() / "data/dummy_image.png"),
            category=DataCategory.PRIVATE,
        )
        logs = str(context)

    assert "Unable to log image metric test_log_image as private, skipping." in logs


def test_logging_aml_metric_list_tuple():
    """Pytest CompliantLogger.metric_list"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()
    log.metric_list(
        name="test_log_list", value=[1, 2, 3, 4], category=DataCategory.PUBLIC
    )
    log.metric_list(
        name="test_log_tuple", value=("1", "2", "test"), category=DataCategory.PUBLIC
    )
    with stream_handler(log, "") as context:
        log.metric_list(
            name="test_log_empty_list", value=[], category=DataCategory.PUBLIC
        )
        log.metric_list(name="test_log_tupe_private", value=("1", "2", "test", None))
        logs = str(context)
    assert "List Value for Metric test_log_empty_list is empty. Skipping." in logs
    assert "ListMetric      | test_log_tupe_private | ['1', '2', 'test', None]" in logs


def test_logging_aml_metric_row():
    """Pytest COmpliantLogger.metric_row"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()

    log.metric_row(
        name="test_row",
        description="stats",
        category=DataCategory.PUBLIC,
        total_size=100,
        file_count=200,
    )

    with stream_handler(log, "") as context:
        log.metric_row(
            name="test_row",
            description="stats",
            category=DataCategory.PRIVATE,
            total_size=100,
            file_count=200,
        )
        logs = str(context)
    assert "RowMetric      | test_row | total_size:100 | file_count:200" in logs


def test_logging_aml_metric_table():
    """Pytest CompliantLogger.metric_table"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()

    test_table1 = {"name": ["James", "Robert", "Michael"], "number": [2, 3, 1, 5]}
    test_table2 = {"name": ["James", "Robert", "Michael"], "number": 2}
    test_table3 = {"name": 2, "number": 4}
    test_table4 = {"name": ["James", "Robert", "Michael"], "number": [2, 3, None]}

    log.metric_table(
        name="test_table1", value=test_table1, category=DataCategory.PUBLIC
    )
    with stream_handler(log, "") as context:
        log.metric_table(name="test_table1", value=test_table1)
        logs = str(context)
    assert "TableMetric     | test_table" in logs
    assert "TableMetric     | Index | name            | number" in logs
    assert "TableMetric     | 00000 | James           | 2              " in logs
    assert "TableMetric     | 00001 | Robert          | 3              " in logs
    assert "TableMetric     | 00002 | Michael         | 1              " in logs
    assert "TableMetric     | 00003 |                 | 5              " in logs

    # Checking empty value
    with stream_handler(log, "") as context:
        log.metric_table(name="empty_input", value={}, category=DataCategory.PUBLIC)
        logs = str(context)
    assert "Dictionary Value for Metric empty_input is empty. Skipping." in logs

    # Checking mixed types
    with stream_handler(log, "") as context:
        log.metric_table(
            name="mixed_type", value=test_table2, category=DataCategory.PUBLIC
        )
        logs = str(context)
    assert (
        "The provided dictionary for metric mixed_type appears to be unstructured!"
        in logs
    )

    log.metric_table(
        name="test_table3", value=test_table3, category=DataCategory.PUBLIC
    )

    with stream_handler(log, "") as context:
        log.metric_table(name="test_table4", value=test_table4)
        logs = str(context)
    assert "TableMetric     | test_table" in logs
    assert "TableMetric     | Index | name            | number" in logs
    assert "TableMetric     | 00000 | James           | 2              " in logs
    assert "TableMetric     | 00001 | Robert          | 3              " in logs
    assert "TableMetric     | 00002 | Michael         |                " in logs


def test_logging_aml_metric_residual():
    """Pytest CompliantLogger.metric_residual"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()

    # Testing vaex dataframe
    test_input1 = vaex.from_arrays(
        x=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], y=[1.1, 2.0, 3.2, 3.8, 5, 6, 7, 8, 9, 10]
    )
    log.metric_residual(
        name="test_log_residual",
        value=test_input1,
        col_predict="x",
        col_target="y",
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_residual(
            name="test_log_residual",
            value=test_input1,
            category=DataCategory.PUBLIC,
        )
    with stream_handler(log, "") as context:
        log.metric_residual(
            name="test_log_residual",
            value=test_input1,
            col_predict="x",
            col_target="y",
        )
        logs = str(context)
    assert "Logging Residuals to text is not yet implemented" in logs

    # Testing spark dataframe
    test_input2 = (
        SparkSession.builder.appName("SparkUnitTests")
        .getOrCreate()
        .createDataFrame(
            data=[(1.0, 1.1), (2.0, 2.0), (3.0, 3.1), (4.0, 3.8), (5.0, 5.2)],
            schema=StructType(
                [
                    StructField("prediction", FloatType(), True),
                    StructField("target", FloatType(), True),
                ]
            ),
        )
    )
    log.metric_residual(
        name="test_log_residual",
        value=test_input2,
        col_predict="prediction",
        col_target="target",
        category=DataCategory.PUBLIC,
    )

    # Testing panda dataframe
    test_input3 = pd.DataFrame(
        [[1.0, 1.1], [2.0, 2.0], [3.0, 3.1], [4.0, 3.8], [100.0, 5.2]],
        columns=["prediction", "target"],
    )
    log.metric_residual(
        name="test_log_residual",
        value=test_input3,
        col_predict="prediction",
        col_target="target",
        category=DataCategory.PUBLIC,
    )

    # Testing un-supported data type
    test_input4 = [1, 2, 3, 4]
    with pytest.raises(PublicRuntimeError):
        log.metric_residual(
            name="test_log_residual",
            value=test_input4,
            col_predict="x",
            col_target="y",
            category=DataCategory.PUBLIC,
        )

    # Testing dict
    test_input5 = {
        "target": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "prediction": [1.1, 2.0, 3.2, 3.8, 5, 6, 7, 8, 9, 10],
    }
    test_input6 = {
        "schema_type": "residuals",
        "schema_version": "1.0.0",
        "data": {
            "bin_edges": [0.0, 0.25, 0.5, 0.75, 1.0],
            "bin_counts": [0.0, 0.0, 0.0, 0.1],
        },
    }
    test_input7 = {
        "target": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "prediction": [1.1],
    }
    log.metric_residual(
        name="test_log_residual",
        value=test_input5,
        col_predict="prediction",
        col_target="target",
        bin_edges=3,
        category=DataCategory.PUBLIC,
    )
    log.metric_residual(
        name="test_log_residual",
        value=test_input6,
        col_predict="prediction",
        col_target="target",
        bin_edges=4,
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_residual(
            name="test_log_residual",
            value=test_input7,
            col_predict="prediction",
            col_target="target",
            category=DataCategory.PUBLIC,
        )


def test_logging_aml_metric_predictions():
    """Pytest CompliantLogger.metric_predictions"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()

    # Testing vaex dataframe
    test_input1 = vaex.from_arrays(
        x=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10], y=[1.1, 2.0, 3.2, 3.8, 5, 6, 7, 8, 9, 10]
    )
    log.metric_predictions(
        name="test_log_predictions",
        value=test_input1,
        col_predict="x",
        col_target="y",
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_predictions(
            name="test_log_predictions",
            value=test_input1,
            category=DataCategory.PUBLIC,
        )
    with stream_handler(log, "") as context:
        log.metric_predictions(
            name="test_log_predictions",
            value=test_input1,
            col_predict="x",
            col_target="y",
        )
        logs = str(context)
    assert "Logging Predictions to text is not yet implemented" in logs

    # Testing spark dataframe
    test_input2 = (
        SparkSession.builder.appName("SparkUnitTests")
        .getOrCreate()
        .createDataFrame(
            data=[(1.0, 1.1), (2.0, 2.0), (3.0, 3.1), (4.0, 3.8), (5.0, 5.2)],
            schema=StructType(
                [
                    StructField("prediction", FloatType(), True),
                    StructField("target", FloatType(), True),
                ]
            ),
        )
    )
    log.metric_predictions(
        name="test_log_predictions",
        value=test_input2,
        col_predict="prediction",
        col_target="target",
        category=DataCategory.PUBLIC,
    )

    # Testing panda dataframe
    test_input3 = pd.DataFrame(
        [[1.0, 1.1], [2.0, 2.0], [3.0, 3.1], [4.0, 3.8], [100.0, 5.2]],
        columns=["prediction", "target"],
    )
    log.metric_predictions(
        name="test_log_predictions",
        value=test_input3,
        col_predict="prediction",
        col_target="target",
        category=DataCategory.PUBLIC,
    )

    # Testing un-supported data type
    test_input4 = [1, 2, 3, 4]
    with pytest.raises(PublicRuntimeError):
        log.metric_predictions(
            name="test_log_predictions",
            value=test_input4,
            col_predict="x",
            col_target="y",
            category=DataCategory.PUBLIC,
        )

    # Testing dict
    test_input5 = {
        "target": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "prediction": [1.1, 2.0, 3.2, 3.8, 5, 6, 7, 8, 9, 10],
    }
    test_input6 = {
        "schema_type": "predictions",
        "schema_version": "1.0.0",
        "data": {
            "bin_averages": [1, 2, 3, 4],
            "bin_errors": [0.0, 0.0, 0.0, 0.0],
            "bin_counts": [0, 0, 0, 0],
            "bin_edges": [0.0, 0.25, 0.5, 0.75, 1.0],
        },
    }
    test_input7 = {
        "target": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "prediction": [1.1],
    }
    log.metric_predictions(
        name="test_log_predictions",
        value=test_input5,
        col_predict="prediction",
        col_target="target",
        bin_edges=3,
        category=DataCategory.PUBLIC,
    )
    log.metric_predictions(
        name="test_log_predictions",
        value=test_input6,
        col_predict="prediction",
        col_target="target",
        bin_edges=4,
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_predictions(
            name="test_log_predictions",
            value=test_input7,
            col_predict="prediction",
            col_target="target",
            category=DataCategory.PUBLIC,
        )


def test_logging_aml_metric_confusion_matrix():
    """Pytest CompliantLogger.metric_confusion_matrix"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()

    # Testing vaex dataframe
    test_input1 = vaex.from_arrays(
        x=["cat", "ant", "cat", "cat", "ant", "bird"],
        y=["ant", "ant", "cat", "cat", "ant", "cat"],
    )
    log.metric_confusion_matrix(
        name="test_log_confusion_matrix",
        value=test_input1,
        idx_true="x",
        idx_pred="y",
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_confusion_matrix(
            name="test_log_confusion_matrix",
            value=test_input1,
            category=DataCategory.PUBLIC,
        )
    with stream_handler(log, "") as context:
        log.metric_confusion_matrix(
            name="test_log_confusion_matrix",
            value=test_input1,
            idx_true="x",
            idx_pred="y",
        )
        logs = str(context)
    assert "Logging Confusion Matrices to text is not yet implemented" in logs

    # Testing spark dataframe
    test_input2 = (
        SparkSession.builder.appName("SparkUnitTests")
        .getOrCreate()
        .createDataFrame(
            data=[(1, 1), (2, 2), (1, 2), (2, 2), (1, 2)],
            schema=StructType(
                [
                    StructField("prediction", IntegerType(), True),
                    StructField("target", IntegerType(), True),
                ]
            ),
        )
    )
    log.metric_confusion_matrix(
        name="test_log_confusion_matrix",
        value=test_input2,
        idx_true="target",
        idx_pred="prediction",
        category=DataCategory.PUBLIC,
    )

    # Testing panda dataframe
    test_input3 = pd.DataFrame(
        [[1, 1], [2, 2], [1, 2], [1, 2], [1, 1]],
        columns=["prediction", "target"],
    )
    log.metric_confusion_matrix(
        name="test_log_confusion_matrix",
        value=test_input3,
        idx_true="target",
        idx_pred="prediction",
        category=DataCategory.PUBLIC,
    )

    # Testing un-supported data type
    test_input4 = [1, 2, 3, 4]
    with pytest.raises(PublicRuntimeError):
        log.metric_confusion_matrix(
            name="test_log_confusion_matrix",
            value=test_input4,
            idx_true="target",
            idx_pred="prediction",
            category=DataCategory.PUBLIC,
        )

    # Testing dict
    test_input5 = {
        "target": ["cat", "ant", "cat", "cat", "ant", "bird"],
        "prediction": ["ant", "ant", "cat", "cat", "ant", "cat"],
    }
    test_input6 = {
        "schema_type": "confusion_matrix",
        "schema_version": "1.0.0",
        "data": {
            "class_labels": [1, 2],
            "matrix": [[2, 0], [2, 1]],
        },
    }
    test_input7 = {
        "target": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "prediction": [1.1],
    }
    log.metric_confusion_matrix(
        name="test_log_confusion_matrix",
        value=test_input5,
        idx_true="target",
        idx_pred="prediction",
        category=DataCategory.PUBLIC,
    )
    log.metric_confusion_matrix(
        name="test_log_confusion_matrix",
        value=test_input6,
        idx_true="target",
        idx_pred="prediction",
        labels=["cat", "dog"],
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_confusion_matrix(
            name="test_log_confusion_matrix",
            value=test_input7,
            idx_true="target",
            idx_pred="prediction",
            category=DataCategory.PUBLIC,
        )


def test_logging_aml_metric_accuracy_table():
    """Pytest CompliantLogger.metric_accuracy_table"""
    compliant_logging.enable_compliant_logging(use_aml_metrics=True)
    log = logging.getLogger()

    # Testing vaex dataframe
    test_input1 = vaex.from_arrays(x=[0.1, 0.3, 0.7], y=["a", "b", "c"])
    log.metric_accuracy_table(
        name="test_log_accuracy_table",
        value=test_input1,
        col_predict="x",
        col_target="y",
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_accuracy_table(
            name="test_log_accuracy_table",
            value=test_input1,
            category=DataCategory.PUBLIC,
        )
    with stream_handler(log, "") as context:
        log.metric_accuracy_table(
            name="test_log_accuracy_table",
            value=test_input1,
            col_predict="x",
            col_target="y",
        )
        logs = str(context)
    assert "Logging Accuracy Tables to text is not yet implemented" in logs

    # Testing spark dataframe
    test_input2 = (
        SparkSession.builder.appName("SparkUnitTests")
        .getOrCreate()
        .createDataFrame(
            data=[(0.1, "a"), (0.3, "b"), (0.6, "c")],
            schema=StructType(
                [
                    StructField("probability", FloatType(), True),
                    StructField("labels", StringType(), True),
                ]
            ),
        )
    )
    log.metric_accuracy_table(
        name="test_log_accuracy_table",
        value=test_input2,
        col_predict="probability",
        col_target="labels",
        category=DataCategory.PUBLIC,
    )

    # Testing panda dataframe
    test_input3 = pd.DataFrame(
        [[0.1, "a"], [0.3, "b"], [0.6, "c"]],
        columns=["probability", "labels"],
    )
    log.metric_accuracy_table(
        name="test_log_accuracy_table",
        value=test_input3,
        col_predict="probability",
        col_target="labels",
        category=DataCategory.PUBLIC,
    )

    # Testing un-supported data type
    test_input4 = [1, 2, 3, 4]
    with pytest.raises(PublicRuntimeError):
        log.metric_accuracy_table(
            name="test_log_accuracy_table",
            value=test_input4,
            col_predict="probability",
            col_target="labels",
            category=DataCategory.PUBLIC,
        )

    # Testing dict
    test_input5 = {
        "probability": [0.1, 0.3, 0.7],
        "labels": ["a", "b", "c"],
    }
    test_input6 = {
        "schema_type": "accuracy_table",
        "schema_version": "1.0.1",
        "data": {
            "probability_tables": [
                [[1, 2, 0, 0], [0, 2, 0, 1], [0, 1, 1, 1], [0, 0, 2, 1], [0, 0, 2, 1]],
                [[1, 2, 0, 0], [1, 1, 1, 0], [0, 1, 1, 1], [0, 0, 2, 1], [0, 0, 2, 1]],
                [[1, 2, 0, 0], [1, 1, 1, 0], [1, 0, 2, 0], [0, 0, 2, 1], [0, 0, 2, 1]],
            ],
            "precentile_tables": [
                [[1, 2, 0, 0], [0, 2, 0, 1], [0, 2, 0, 1], [0, 0, 2, 1], [0, 0, 2, 1]],
                [[1, 1, 1, 0], [0, 1, 1, 1], [0, 1, 1, 1], [0, 0, 2, 1], [0, 0, 2, 1]],
                [[1, 0, 2, 0], [0, 0, 2, 1], [0, 0, 2, 1], [0, 0, 2, 1], [0, 0, 2, 1]],
            ],
            "probability_thresholds": [0.0, 0.25, 0.5, 0.75, 1.0],
            "percentile_thresholds": [0.0, 0.01, 0.24, 0.98, 1.0],
            "class_labels": ["a", "b", "c"],
        },
    }
    test_input7 = {
        "probability": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "labels": [1.1],
    }
    log.metric_accuracy_table(
        name="test_log_accuracy_table",
        value=test_input5,
        col_predict="probability",
        col_target="labels",
        category=DataCategory.PUBLIC,
    )
    log.metric_accuracy_table(
        name="test_log_accuracy_table",
        value=test_input6,
        col_predict="probability",
        col_target="labels",
        category=DataCategory.PUBLIC,
    )
    with pytest.raises(PublicRuntimeError):
        log.metric_accuracy_table(
            name="test_log_accuracy_table",
            value=test_input7,
            col_predict="probability",
            col_target="labels",
            category=DataCategory.PUBLIC,
        )


def test_convert_obj():
    """Pytest CompliantLogger._convert_obj"""
    logger = CompliantLogger(name="")
    assert logger._convert_obj("tests", category=DataCategory.PUBLIC) == "tests"

    assert "Spark DataFrame (Row Count: 5 / Column Count: 2)" in logger._convert_obj(
        (
            SparkSession.builder.appName("SparkUnitTests")
            .getOrCreate()
            .createDataFrame(
                data=[(1, 1), (2, 2), (1, 2), (2, 2), (1, 2)],
                schema=StructType(
                    [
                        StructField("prediction", IntegerType(), True),
                        StructField("target", IntegerType(), True),
                    ]
                ),
            )
        ),
        category=DataCategory.PUBLIC,
    )

    assert "Vaex DataFrame (Row Count: 5 / Column Count: 2)" in logger._convert_obj(
        vaex.from_arrays(x=np.arange(5), y=np.arange(5) ** 2)
    )
    test_df = pd.DataFrame(
        [["Tom", 10], ["Nick", 15], ["John", 14]], columns=["Name", "Age"]
    )
    assert "Pandas DataFrame (Row Count: 3 / Column Count: 2)" in logger._convert_obj(
        test_df
    )

    test_series = pd.Series([1, 2, 3, 4, 5])
    assert "Pandas Series (Row Count: 5)" in logger._convert_obj(test_series)

    test_ndarray = np.array([1, 2, 3, 4, 5])
    assert "Numpy Array (Shape: (5,))" in logger._convert_obj(test_ndarray)

    assert "List (Count: 5) | range(0, 5)" in logger._convert_obj(
        range(5), category=DataCategory.PUBLIC
    )
    assert "List (Count: 10) | range(0, 10)..." in logger._convert_obj(
        range(10), category=DataCategory.PUBLIC
    )


@pytest.mark.parametrize(
    "tenant_id, subscription_id, is_eyesoff_workspace",
    [
        (
            "cdc5aeea-15c5-4db6-b079-fcadd2505dc2",
            "60d27411-7736-4355-ac95-ac033929fe9d",
            True,
        ),
        (
            "cdc5aeea-15c5-4db6-b079-fcadd2505dc2",
            "2ffad2ef-10c3-4793-aacc-093dc4ad2e63",
            False,
        ),
        (
            "72f988bf-86f1-41af-91ab-2d7cd011db47",
            "48bbc269-ce89-4f6f-9a12-c6f91fcb772d",
            False,
        ),
    ],
)
def test_is_eyesoff(tenant_id, subscription_id, is_eyesoff_workspace):
    with patch(
        "os.environ",
        {
            "AZ_BATCHAI_CLUSTER_TENANT_ID": tenant_id,
            "AZ_BATCHAI_CLUSTER_SUBSCRIPTION_ID": subscription_id,
        },
    ):
        assert is_eyesoff() == is_eyesoff_workspace


def test_is_eyesoff_empty():
    assert not is_eyesoff()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
from shrike._core import experimental
import logging

log = logging.getLogger(__name__)


def test_experimental_with_custom_msg(caplog):
    @experimental(message="custom message")
    def foo():
        log.warn("This is foo.")

    with caplog.at_level("WARN"):
        foo()
    assert "custom message" in caplog.text
    assert "This is foo." in caplog.text


def test_experimental_with_default_msg(caplog):
    @experimental()
    def foo(a):
        return a

    with caplog.at_level("WARN"):
        x = foo(1)
    assert x == 1
    assert (
        "This is an experimental feature and could change at any time." in caplog.text
    )



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Unit tests for the `prepare` module.

Consider decorating long end-to-end tests with `@pytest.mark.order(-1)`.
"""

import pytest
import os
import copy
from pathlib import Path
import subprocess
import shutil
import yaml
from git import Repo
from unittest import mock

from shrike.build.commands import prepare
from shrike.build.core.configuration import (
    Configuration,
    load_configuration_from_args_and_env,
)

TESTING_WORKSPACE = "/subscriptions/48bbc269-ce89-4f6f-9a12-c6f91fcb772d/resourceGroups/github-ci-rg/providers/Microsoft.MachineLearningServices/workspaces/github-ci-ml-wus2"

SPEC_YAML = {
    "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
    "name": "dummy",
    "version": "0.0.1",
    "type": "CommandComponent",
    "command": "pip freeze",
    "environment": {
        "conda": {"conda_dependencies_file": "conda_env.yaml"},
        "docker": {
            "image": "polymerprod.azurecr.io/polymercd/prod_official/azureml_base_gpu_openmpi312cuda101cudnn7_mcr:latest"
        },
    },
}

ENV_YAML = {
    "name": "dummy_env",
    "channels": ["."],
    "dependencies": [
        "python=3.7",
        {
            "pip": [
                "azureml-core==1.20.0",
                "shrike",
                "numpy==1.19.4",
                "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
            ]
        },
    ],
}


def clean() -> None:
    """
    Clean up all non checked-in files.
    """
    subprocess.run(["git", "clean", "-xdf"])


@pytest.mark.order(-1)
def test_build_all_components(caplog):
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="foo")
    prep.ensure_component_cli_installed()
    component = "tests/tests_build/steps/component1/spec.yaml"
    with caplog.at_level("INFO"):
        success = prep.build_all_components([component])
    assert success
    assert f"Component {component} is built." in caplog.text


@pytest.mark.parametrize("mode", ["foo", "aether", "aml"])
def test_find_component_specification_files_using_all(mode):
    # clean the .build directories first, such that we won't include the
    # spec.yaml files under .build directories in the unit tests here
    clean()

    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode=mode)

    dir = str(Path(__file__).parent.parent.resolve() / "steps/component1")
    print(dir)
    res = prep.find_component_specification_files_using_all(dir=dir)
    print("found spec yaml file paths list: ", res)
    assert len(res) == 1

    dir = str(Path(__file__).parent.parent / "steps/component1")
    print(dir)
    res = prep.find_component_specification_files_using_all(dir=dir)
    print("found spec yaml file paths list: ", res)
    assert len(res) == 1

    dir = str(Path(__file__).parent.parent.resolve() / "steps/component2")
    print(dir)
    res = prep.find_component_specification_files_using_all(dir=dir)
    print("found spec yaml file paths list: ", res)
    assert len(res) == 1

    dir = str(Path(__file__).parent.parent / "steps/component2")
    print(dir)
    res = prep.find_component_specification_files_using_all(dir=dir)
    print("found spec yaml file paths list: ", res)
    assert len(res) == 1

    dir = str(Path(__file__).parent.parent.resolve() / "steps/component3")
    print(dir)
    res = prep.find_component_specification_files_using_all(dir=dir)
    print("found spec yaml file paths list: ", res)
    assert len(res) == 1

    dir = str(Path(__file__).parent.parent / "steps/component3")
    print(dir)
    res = prep.find_component_specification_files_using_all(dir=dir)
    print("found spec yaml file paths list: ", res)
    assert len(res) == 1


def test_create_catalog_files_fails_if_non_standard_mode():
    with pytest.raises(ValueError):
        prep = prepare.Prepare()
        prep.config = Configuration(signing_mode="foo")
        prep.create_catalog_files([""])


def test_create_catalog_files_fails_if_non_standard():
    with pytest.raises(ValueError):
        prep = prepare.Prepare()
        prep.config = Configuration(signing_mode="foo")
        prep.create_catalog_files(
            [
                "tests/tests_build/steps/component1/spec.yaml",
                "tests/tests_build/steps/component2/spec.yaml",
            ]
        )


def test_non_standard_activation_method_not_supported():
    with pytest.raises(ValueError):
        prep = prepare.Prepare()
        prep.config = Configuration(activation_method="foo")
        prep.find_component_specification_files()


def test_smart_method_is_supported():
    prep = prepare.Prepare()
    prep.config = Configuration(activation_method="smart")
    prep.find_component_specification_files()


def test_create_catalog_files_for_aether(caplog):
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="aether")
    prep.ensure_component_cli_installed()
    component = [
        "tests/tests_build/steps/component1/spec.yaml",
        "tests/tests_build/steps/component2/spec.yaml",
    ]
    with caplog.at_level("INFO"):
        prep.create_catalog_files_for_aether(component)
    assert "Finish creating aether catalog files for component1." in caplog.text
    assert "Finish creating aether catalog files for component2." in caplog.text
    assert os.path.exists("tests/tests_build/steps/component1/component1.cat")
    assert os.path.exists("tests/tests_build/steps/component2/component2.cat")
    os.remove("tests/tests_build/steps/component1/component1.cat")
    os.remove("tests/tests_build/steps/component2/component2.cat")


@pytest.mark.parametrize(
    "modified_file,component",
    [
        (
            "./tests/tests_build/steps/component1/another_file_for_component1.txt",
            "./tests/tests_build/steps/component1/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/component2/subdir1/subsubdir1/file_in_subsubdir1.txt",
            "./tests/tests_build/steps/component2/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/component2/subdir2/file_in_subdir2.txt",
            "./tests/tests_build/steps/component2/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/component2/subdir2/some_deleted_file.txt",
            "./tests/tests_build/steps/component2/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/component2/some_deleted_subdir/some_deleted_file.txt",
            "./tests/tests_build/steps/component2/spec.yaml",
        ),
    ],
)
def test_is_in_subfolder(modified_file, component):
    prep = prepare.Prepare()
    res = prep.is_in_subfolder(modified_file, component)
    assert res


@pytest.mark.parametrize(
    "modified_file,component",
    [
        (
            "./tests/tests_build/steps/component1/another_file_for_component1.txt",
            "./tests/tests_build/steps/component2/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/component2/subdir1/subsubdir1/file_in_subsubdir1.txt",
            "./tests/tests_build/steps/component1/spec.yaml",
        ),
        (
            "./tests/tests_build/commands/test_prepare.py",
            "./tests/tests_build/steps/component1/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/component1/some_deleted_file.txt",
            "./tests/tests_build/steps/component2/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/deleted_component/subdir/some_deleted_file.txt",
            "./tests/tests_build/steps/component2/spec.yaml",
        ),
        (
            "./tests/tests_build/steps/component2/subdir2/file_in_subdir2.txt",
            "./tests/tests_build/steps/deleted_component/spec.yaml",
        ),
    ],
)
def test_is_not_in_subfolder(modified_file, component):
    prep = prepare.Prepare()
    res = prep.is_in_subfolder(modified_file, component)
    assert res == False


@pytest.mark.parametrize(
    "modified_file,component_additional_includes_contents",
    [
        (
            "./shrike/build/commands/prepare.py",
            [
                "./shrike/build/commands",
                "./shrike/build/non_existent_directory",
            ],
        ),
        (
            "./shrike/build/commands/some_deleted_file.txt",
            [
                "./shrike/build/commands",
                "./shrike/build/non_existent_directory",
            ],
        ),
        (
            "./shrike/build/commands/prepare.py",
            [
                "./shrike/build/commands/prepare.py",
                "./shrike/build/non_existent_directory",
            ],
        ),
        (
            "./shrike/build/commands/prepare.py",
            ["./shrike/build", "./shrike/build/non_existent_directory"],
        ),
    ],
)
def test_is_in_additional_includes(
    modified_file, component_additional_includes_contents
):
    prep = prepare.Prepare()
    res = prep.is_in_additional_includes(
        modified_file, component_additional_includes_contents
    )
    assert res


@pytest.mark.parametrize(
    "modified_file,component_additional_includes_contents",
    [
        (
            "./shrike/build/commands/prepare.py",
            [
                "./shrike/build/core",
                "./shrike/build/tests",
                "./shrike/build/non_existent_directory",
            ],
        ),
        (
            "./shrike/build/commands/some_deleted_file.txt",
            [
                "./shrike/build/core",
                "./shrike/build/tests",
                "./shrike/build/non_existent_directory",
                "./shrike/build/non_existent_directory/non_existent_subdirectory",
            ],
        ),
        (
            "./shrike/build/commands/some_deleted_directory/some_deleted_file.txt",
            [
                "./shrike/build/core",
                "./shrike/build/tests",
                "./shrike/build/non_existent_directory",
            ],
        ),
        (
            "./shrike/build/commands/some_deleted_directory/some_deleted_subsirectory/some_deleted_file.txt",
            [
                "./shrike/build/core",
                "./shrike/build/tests",
                "./shrike/build/non_existent_directory",
            ],
        ),
        (
            "./shrike/build/__init__.py",
            [
                "./shrike/build/core",
                "./shrike/build/tests",
                "./shrike/build/non_existent_directory",
            ],
        ),
    ],
)
def test_is_not_in_additional_includes(
    modified_file, component_additional_includes_contents
):
    prep = prepare.Prepare()
    res = prep.is_in_additional_includes(
        modified_file, component_additional_includes_contents
    )
    assert res == False


@pytest.mark.parametrize(
    "component,modified_files",
    [
        (
            "./tests/tests_build/steps/component1/spec.yaml",
            [
                "./shrike/build/commands/prepare.py",
            ],
        ),
        (
            "./tests/tests_build/steps/component1/spec.yaml",
            [
                "./tests/tests_build/commands/test_prepare.py",
            ],
        ),
        (
            "./tests/tests_build/steps/component2/spec.yaml",
            [
                "./tests/tests_build/steps/component2/subdir1/subsubdir1/file_in_subsubdir1.txt",
            ],
        ),
        (
            "./tests/tests_build/steps/component2/spec.yaml",
            [
                "./tests/tests_build/steps/component2/subdir2/file_in_subsubdir2.txt",
            ],
        ),
        (
            "./tests/tests_build/steps/component3/spec.yaml",
            [
                "./tests/tests_build/steps/component3/spec.yaml",
            ],
        ),
        (
            "./tests/tests_build/steps/component3/spec.yaml",
            [
                "./tests/tests_build/steps/component3/some_deleted_file.txt",
            ],
        ),
    ],
)
def test_component_is_active(component, modified_files):
    prep = prepare.Prepare()
    res = prep.component_is_active(component, modified_files)
    assert res


@pytest.mark.parametrize(
    "component,modified_files",
    [
        (
            "./tests/tests_build/steps/deleted_component/spec.yaml",
            [
                "./shrike/build/commands/prepare.py",
            ],
        ),
        (
            "./tests/tests_build/steps/component1/spec.yaml",
            [
                "./tests/tests_build/steps/deleted_component/deleted_file.txt",
            ],
        ),
    ],
)
def test_component_is_active_for_deleted_component_or_file(component, modified_files):
    prep = prepare.Prepare()
    res = prep.component_is_active(component, modified_files)
    assert res == False


@pytest.mark.parametrize(
    "component,modified_files",
    [
        (
            "./tests/tests_build/steps/component2/spec.yaml",
            [
                "./shrike/build/commands/prepare.py",
            ],
        ),
        (
            "./tests/tests_build/steps/component2/spec.yaml",
            [
                "./tests/tests_build/commands/test_prepare.py",
            ],
        ),
        (
            "./tests/tests_build/steps/component2/spec.yaml",
            [
                "./tests/tests_build/steps/component1/spec.yaml",
            ],
        ),
    ],
)
def test_component_is_not_active(component, modified_files):
    prep = prepare.Prepare()
    res = prep.component_is_active(component, modified_files)
    assert res == False


@pytest.mark.parametrize(
    "modified_files, expected_res",
    [
        (
            [
                "./shrike/build/commands/prepare.py",
            ],
            [
                "./tests/tests_build/steps/component1/spec.yaml",
            ],
        ),
        (
            [
                "./tests/tests_build/commands/test_prepare.py",
            ],
            [
                "./tests/tests_build/steps/component1/spec.yaml",
            ],
        ),
        (
            [
                "./tests/tests_build/steps/component2/subdir1/subsubdir1/file_in_subsubdir1.txt",
            ],
            [
                "./tests/tests_build/steps/component2/spec.yaml",
            ],
        ),
        (
            [
                "./tests/tests_build/steps/component2/subdir2/file_in_subdir2.txt",
                "./tests/tests_build/steps/component3/spec.yaml",
            ],
            [
                "./tests/tests_build/steps/component2/spec.yaml",
                "./tests/tests_build/steps/component3/spec.yaml",
            ],
        ),
    ],
)
def test_infer_active_components_from_modified_files(modified_files, expected_res):
    prep = prepare.Prepare()
    prep.config = Configuration()
    res = prep.infer_active_components_from_modified_files(modified_files)
    for res_line_number in range(0, len(res)):
        assert res[res_line_number] == str(
            Path(expected_res[res_line_number]).resolve()
        )
    assert len(res) == len(expected_res)


def test_get_modified_files():

    # Creating a new repo and declaring branch names
    main_branch_name = "main"
    new_branch_name = "newbranch"
    root_repo_location = "./temp_repo/"
    if Path(root_repo_location).exists():
        shutil.rmtree(root_repo_location)
    repo_path = Path(root_repo_location) / "bare-repo"
    print("New repo path: " + str(repo_path.resolve()))
    new_repo = Repo.init(repo_path, initial_branch="main")

    # First commit to main
    # creating some files
    file_name_1 = repo_path / "new-file-1.txt"
    open(file_name_1, "wb").close()
    file_name_2 = repo_path / "new-file-2.py"
    open(file_name_2, "wb").close()
    subdir_path = repo_path / "subdirectory"
    try:
        os.mkdir(subdir_path)
    except:
        print(str(subdir_path) + " already exists; no need to create it.")
    file_name_3 = subdir_path / "new-file-3.yaml"
    open(file_name_3, "wb").close()
    # add them to the index
    new_repo.index.add(
        [
            str(file_name_1.resolve()),
            str(file_name_2.resolve()),
            str(file_name_3.resolve()),
        ]
    )  #
    # do the commit
    new_repo.index.commit("Merged PR: First one")

    # Second commit to main
    # create a new file
    file_name_4 = repo_path / "new-file-4.py"
    open(file_name_4, "wb").close()
    # modify an old file
    with open(file_name_1, "w") as file:
        file.write("This is a change to the first file\n")
    new_repo.index.add([str(file_name_1.resolve()), str(file_name_4.resolve())])
    new_repo.index.commit("Merged PR: Second one")

    # create a "remote" and push everything to it (a remote is needed for the RB case)
    remote_repo_path_string = "../remote-repo"  # relative to 'repo_path'
    print("Remote repo path: " + str(remote_repo_path_string))
    cloned_repo = new_repo.clone(remote_repo_path_string)
    new_repo.create_remote("origin", url=remote_repo_path_string)
    new_repo.remotes.origin.pull(refspec=main_branch_name + ":origin")

    # Third commit, to a different branch and with a file deletion
    file_name_5 = repo_path / "new-file-5.py"
    open(file_name_5, "wb").close()
    # modify an old file
    with open(file_name_2, "w") as file:
        file.write("This is a change to the second file\n")
    # delete a file
    os.remove(file_name_3.resolve())
    new_repo.git.checkout("HEAD", b=new_branch_name)
    new_repo.index.add([str(file_name_2.resolve()), str(file_name_5.resolve())])
    new_repo.index.remove([str(file_name_3.resolve())])
    new_repo.index.commit("This is a commit in the non-compliant branch")

    # now we're ready to do the actual tests
    prep = prepare.Prepare()
    prep.config = Configuration()
    # 1. test the 'Build - after Merge' case (BAM)
    change_list_BAM = prep.get_modified_files(
        new_repo, main_branch_name, main_branch_name
    )
    assert change_list_BAM == {
        str(file_name_1.resolve()),
        str(file_name_4.resolve()),
    }
    # 2. test the 'Build - before Merge' case (BBM)
    change_list_BBM = prep.get_modified_files(
        new_repo, "refs/pull/XXXXXX/merge", main_branch_name
    )
    assert change_list_BBM == {
        str(file_name_2.resolve()),
        str(file_name_3.resolve()),
        str(file_name_5.resolve()),
    }
    # 3. test the 'Manual' case (triggered from local)
    change_list_Manual = prep.get_modified_files(
        new_repo, new_branch_name, main_branch_name
    )
    assert change_list_Manual == {
        str(file_name_2.resolve()),
        str(file_name_3.resolve()),
        str(file_name_5.resolve()),
    }

    # 4. test the 'Manual' case triggered from DevOps
    # (using a non existing branch name to mimic that DevOps case where the repo has detached head and we cannot just take repo.heads[current_branch])
    change_list_Manual = prep.get_modified_files(
        new_repo, "non-existing_branch_name", main_branch_name
    )
    assert change_list_Manual == {
        str(file_name_2.resolve()),
        str(file_name_3.resolve()),
        str(file_name_5.resolve()),
    }

    # clean up the newly created repos
    new_repo.close()
    cloned_repo.close()
    shutil.rmtree(root_repo_location)


@pytest.mark.parametrize(
    "true_commit_message", ["Merged PR: First one", "Merged PR: Second one"]
)
def test_get_compliant_commit_corresponding_to_pull_request(true_commit_message):

    # Create_repo and do 2 commits

    # Creating a new repo
    compliant_branch = "main"
    root_repo_location = "./temp_repo/"
    if Path(root_repo_location).exists():
        shutil.rmtree(root_repo_location)
    repo_path = Path(root_repo_location) / "bare-repo"
    print("New repo path: " + str(repo_path.resolve()))
    new_repo = Repo.init(repo_path, initial_branch="main")

    # First commit
    # creating a file
    file_name_1 = repo_path / "new-file-1.txt"
    open(file_name_1, "wb").close()
    # add it to the index
    new_repo.index.add([str(file_name_1.resolve())])
    # do the commit
    new_repo.index.commit("Merged PR: First one")

    # Second commit to main
    # create a new file
    file_name_2 = repo_path / "new-file-2.py"
    open(file_name_2, "wb").close()
    new_repo.index.add([str(file_name_2.resolve())])
    new_repo.index.commit("Merged PR: Second one")

    # create a "remote" and push everything to it (a remote is needed for the RB case)
    remote_repo_path_string = "../remote-repo"  # relative to 'repo_path'
    print("Remote repo path: " + str(remote_repo_path_string))
    cloned_repo = new_repo.clone(remote_repo_path_string)
    new_repo.create_remote("origin", url=remote_repo_path_string)
    new_repo.remotes.origin.pull(refspec=compliant_branch + ":origin")

    # call the function to test and do some assertions
    prep = prepare.Prepare()

    with mock.patch(
        "shrike.build.prepare.Prepare.get_true_commit_message",
        return_value=true_commit_message,
    ):
        commit = prep.get_compliant_commit_corresponding_to_pull_request(
            cloned_repo, compliant_branch
        )
        assert commit.message == true_commit_message

    # clean up the newly created repos
    new_repo.close()
    cloned_repo.close()
    shutil.rmtree(root_repo_location)


def test_identify_repo_and_branches():
    prep = prepare.Prepare()
    prep.config = Configuration(compliant_branch="^refs/heads/CompliantBranchName$")
    [repo, current_branch, compliant_branch] = prep.identify_repo_and_branches()
    assert repo.bare == False
    assert current_branch
    assert compliant_branch == "CompliantBranchName"


def test_identify_repo_and_branches_when_no_repo():
    prep = prepare.Prepare()
    prep.config = Configuration(
        compliant_branch="^refs/heads/CompliantBranchName$",
        working_directory=str(Path(__file__).parents[4]),
    )  # we set the working directory above the repo level, so there's no repo to be found
    print("Working directory: " + str(prep.config.working_directory))
    with pytest.raises(Exception):
        [repo, current_branch, compliant_branch] = prep.identify_repo_and_branches()


@pytest.mark.order(-1)
def test_ensure_component_cli_installed(caplog):
    prep = prepare.Prepare()
    prep.config = Configuration(activation_method="foo")
    with caplog.at_level("INFO"):
        first_try = prep.ensure_component_cli_installed()
        second_try = prep.ensure_component_cli_installed()

    assert second_try == True
    assert "component CLI exists. Skipping installation." in caplog.text


@pytest.mark.order(-1)
def test_log_info_component_cli_installed(caplog):
    prep = prepare.Prepare()
    prep.config = Configuration(activation_method="foo")
    with caplog.at_level("ERROR"):
        prep.ensure_component_cli_installed()
    assert "Command failed with exit code" not in caplog.text


@pytest.mark.parametrize("catalog_file_name", ["catalog.json", "catalog.json.sig"])
def test_create_catalog_files_for_aml(catalog_file_name):
    prep = prepare.Prepare()
    prep.config = Configuration(suppress_adding_repo_pr_tags=True)
    # we'll test 2 component folder structures
    component1_folder = "tests/tests_build/steps/component1/"  # flat directory
    component2_folder = "tests/tests_build/steps/component2/"  # nested directories
    # create the catalogs
    prep.create_catalog_files_for_aml(
        [
            os.path.join(component1_folder, "spec.yaml"),
            os.path.join(component2_folder, "spec.yaml"),
        ]
    )
    # grab the freshly created catalogs
    component1_catalog_path = os.path.join(component1_folder, catalog_file_name)
    with open(component1_catalog_path, "r") as component1_catalog_file:
        component1_catalog_contents = component1_catalog_file.read()
    component2_catalog_path = os.path.join(component2_folder, catalog_file_name)
    with open(component2_catalog_path, "r") as component2_catalog_file:
        component2_catalog_contents = component2_catalog_file.read()
    # hard-coded values to check
    component1_catalog_reference = '{"HashAlgorithm": "SHA256", "CatalogItems": {"another_file_for_component1.txt": "9E640A18FC586A6D87716F9B4C6728B7023819E58E07C4562E0D2C14DFC3CF5B", "spec.additional_includes": "50407DAA1E6DA1D91E1CE88DDDF18B3DFDA62E08B780EC9B2E8642536DD36C05", "spec.yaml": "BF41A2F4D427A281C0E7EB5F987E285D69F5D50AEFCBB559DC2D7611D861D7FA"}}'
    # component2_catalog_reference = '{"HashAlgorithm": "SHA256", "CatalogItems": {".amlignore": "1CA13EBDBB24D532673E22A0886631976CDC9B9A94488FE31AF9214F4A79E8AE", ".subdir/.gitignore": "A5270F91138FC2BB5470ECB521DAB043140D7E0FD8CB33BB0644AC13EFB60FE7", "another_file_for_component2.txt": "A47275ACA3BC482FC3F4C922572EA514D0DE03EA836597D34FC21BA805D2ABCA", "spec.yaml": "B1CBC48FFB11EBC9C3C84CD0F6BF852EF9573C3C34E83DCD165FF66074B19DFF", "subdir1/.gitignore": "C0159813AB6EAF3CC8A0BD37C79E4CDD927E3E95CB9BA8EC246BC3A176C3EB41", "subdir1/file_in_subdir1.txt": "A7917FCCF0C714716F308967DB45B2DDEE4665FC4B4FCC6C0E50ABD55DD1C6B5", "subdir1/subsubdir1/file_in_subsubdir1.txt": "1791DD6583A06429603CC30CDC2AE6A217853722C6BB10AA31027F5A931D5A7D", "subdir2/file_in_subdir2.txt": "419EE822D1E34B22FCE7F09EDCFC7565188A1362352E1DADB569820CB599D651"}}'
    # assertions
    assert component1_catalog_contents == component1_catalog_reference
    # Note: The hash values of component2 always change after we have a big PR.
    # I tentatively commented it, but may uncomment it in the future.
    # assert component2_catalog_contents == component2_catalog_reference


def test_validate_all_components_does_nothing_if_no_files(caplog):
    prep = prepare.Prepare()

    with caplog.at_level("INFO"):
        prep.validate_all_components([])

    assert not caplog.text


@pytest.mark.order(-1)
def test_validate_all_components_works_on_invalid_component():
    component_path = str(
        Path(__file__).parent.parent.resolve() / "steps/component1/spec.yaml"
    )

    prep = prepare.Prepare()
    prep.config = Configuration()

    prep.attach_workspace(TESTING_WORKSPACE)
    success = prep.validate_all_components([component_path])

    assert not success
    assert prep._component_statuses[component_path]["validate"] == "failed"
    assert len(prep._errors) == 1


@pytest.mark.order(-1)
def test_validate_all_components_works_on_valid_component(caplog):
    component_path = str(
        Path(__file__).parent.parent.resolve() / "steps/component2/spec.yaml"
    )

    prep = prepare.Prepare()
    prep.config = Configuration()

    prep.attach_workspace(TESTING_WORKSPACE)

    with caplog.at_level("INFO"):
        prep.validate_all_components([component_path])

    assert prep._component_statuses[component_path]["validate"] == "succeeded"
    assert not prep._errors
    assert "is valid" in caplog.text


@pytest.mark.order(-1)
def test_validate_all_components_code_snapshot_parameter(caplog):
    tmp_dir = str(Path(__file__).parent.parent.resolve() / "steps/tmp_dir")
    tmp_yaml = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_component",
        "version": "0.0.1",
        "type": "CommandComponent",
        "command": "pip freeze",
        "code": "../../",
    }
    os.mkdir(tmp_dir)
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)

    prep = prepare.Prepare()
    prep.config = Configuration()

    prep.attach_workspace(TESTING_WORKSPACE)

    with caplog.at_level("INFO"):
        prep.validate_all_components([tmp_dir + "/spec.yaml"])

    assert prep._component_statuses[tmp_dir + "/spec.yaml"]["validate"] == "failed"
    assert len(prep._errors) == 1
    assert (
        "Code snapshot parameter is not supported. Please use .additional_includes for your component."
        in caplog.text
    )

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_validate_all_components_code_Section(caplog):
    component_path = str(
        Path(__file__).parent.parent.resolve() / "steps/component4/spec.yaml"
    )

    prep = prepare.Prepare()
    prep.config = Configuration()

    prep.attach_workspace(TESTING_WORKSPACE)

    with caplog.at_level("INFO"):
        prep.validate_all_components([component_path])

    assert prep._component_statuses[component_path]["validate"] == "succeeded"
    assert not prep._errors
    assert "is valid" in caplog.text
    assert (
        "Code snapshot parameter is not supported. Please use .additional_includes for your component."
        not in caplog.text
    )


@pytest.mark.order(-1)
def test_workspace_attachment_for_invalid_workspace(caplog):
    prep = prepare.Prepare()
    prep.config = Configuration()

    # try to attach a fake (non-ecxisting) workspace. expected to receive a register_error message.
    fake_workspace_id = "/subscriptions/48bbc269-ce89-4f6f-9a12-c6f91fcb772d/resourceGroups/aml1p-rg/providers/Microsoft.MachineLearningServices/workspaces/aml1p-ml-canary-fake"
    with caplog.at_level("INFO"):
        prep.attach_workspace(fake_workspace_id)

    assert f"Error!! Failed to attach to {fake_workspace_id}!" in prep._errors
    assert f"Error!! Failed to attach to {fake_workspace_id}!" in caplog.text


@pytest.mark.order(-1)
def test_run_with_config_runs_end_to_end(caplog):
    # clean the .build directories first, such that we won't include the
    # spec.yaml files under .build directories in the unit tests here
    clean()

    prep = prepare.Prepare()
    prep.config = Configuration(workspaces=[TESTING_WORKSPACE])

    with caplog.at_level("INFO"):
        prep.run_with_config()

    assert "Running component preparation logic" in caplog.text


@pytest.mark.parametrize(
    "component,expected_len", [("component1", 3), ("component2", 8), ("component3", 1)]
)
def test_all_files_in_snapshot(component, expected_len):
    clean()

    directory = str(Path(__file__).parent.parent.resolve() / f"steps/{component}")
    prep = prepare.Prepare()
    result = prep.all_files_in_snapshot(f"{directory}/spec.yaml")
    assert len(result) == expected_len


@pytest.mark.order(-1)
def test_add_repo_and_last_pr_to_tags():
    component_path = str(
        Path(__file__).parent.parent.resolve() / "steps/component4/spec.yaml"
    )
    prep = prepare.Prepare()
    prep.config = Configuration()

    prep.add_repo_and_last_pr_to_tags([component_path])
    with open(component_path) as f:
        spec_file = yaml.safe_load(f)
    spec_tags = spec_file.get("tags")
    assert "repo" in spec_tags
    assert "last_commit_id" in spec_tags

    assert "last_commit_message" in spec_tags
    assert "path_to_component" in spec_tags
    assert "[link to commit]" in spec_file.get("description")


def test_add_repo_and_last_pr_to_tags_when_not_in_git():
    clean()
    tmp_dir = str(Path(__file__).parent.parent.resolve() / "steps/tmp_dir")
    os.mkdir(tmp_dir)

    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(SPEC_YAML, tmp_spec)
    new_component_path = tmp_dir + "/spec.yaml"

    prep = prepare.Prepare()
    prep.config = Configuration()
    with pytest.raises(StopIteration):
        prep.add_repo_and_last_pr_to_tags([new_component_path])

    shutil.rmtree(tmp_dir)


def test_extract_python_package_dependencies():
    clean()
    prep = prepare.Prepare()
    prep.config = Configuration()

    conda_dependencies = {
        "name": "democomponent_env",
        "dependencies": [
            "python=3.7",
            {
                "pip": [
                    "azureml-core==1.20.0",
                    "shrike",
                    "numpy==1.19.4",
                    "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
                ]
            },
        ],
    }
    extraction = prep._extract_python_package_dependencies(conda_dependencies)
    assert len(extraction) == 4
    assert extraction == [
        "azureml-core==1.20.0",
        "shrike",
        "numpy==1.19.4",
        "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
    ]


def test_create_requirements_files(caplog):
    clean()
    prep = prepare.Prepare()
    prep.config = Configuration()

    steps_dir = str(Path(__file__).parent.parent.resolve() / "steps")
    component_files = [
        steps_dir + "/component" + str(i) + "/spec.yaml" for i in range(1, 5)
    ]
    with caplog.at_level("INFO"):
        id = prep._create_requirements_files(component_files)
    assert "Writing Python package dependencies to path" in caplog.text
    component_dependencies_repo = "component_dependencies_" + id
    assert os.path.exists(component_dependencies_repo)


def test_create_requirements_file_with_duplicate_component_name(caplog):
    clean()
    prep = prepare.Prepare()
    prep.config = Configuration()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.parent.resolve() / "steps/tmp_dir")
    os.mkdir(tmp_dir)

    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(SPEC_YAML, tmp_spec)
    with open(tmp_dir + "/spec_dup.yaml", "w") as tmp_spec:
        yaml.dump(SPEC_YAML, tmp_spec)
    with open(tmp_dir + "/conda_env.yaml", "w") as tmp_spec:
        yaml.dump(ENV_YAML, tmp_spec)

    component_files = [tmp_dir + "/spec.yaml", tmp_dir + "/spec_dup.yaml"]
    with caplog.at_level("INFO"):
        id = prep._create_requirements_files(component_files)
    component_dependencies_repo = "component_dependencies_" + id
    assert os.path.exists(component_dependencies_repo + "/dummy/requirements.txt")
    assert os.path.exists(
        component_dependencies_repo + "/dummy_spec_dup/requirements.txt"
    )

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


def test_create_requirements_file_for_single_component_conda_dependencies(caplog):
    clean()
    prep = prepare.Prepare()
    prep.config = Configuration()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.parent.resolve() / "steps/tmp_dir")
    os.mkdir(tmp_dir)
    tmp_yaml = {
        "name": "dummy_1",
        "display_name": "C# (.NET Core) component",
        "version": "0.0.0",
        "type": "CommandComponent",
        "command": "chmod +x ./csharp_component && ./csharp_component\n",
        "environment": {
            "docker": {
                "image": "polymerprod.azurecr.io/polymercd/prod_official/azureml_base_cpu:latest"
            },
            "conda": {
                "conda_dependencies": {
                    "dependencies": [
                        "python=3.7",
                        {
                            "pip": [
                                "azureml-core==1.27.0",
                                "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
                            ]
                        },
                    ]
                }
            },
        },
    }
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)

    path_to_requirements_files = (
        prep.config.working_directory + "/tmp_path_to_requirements_files"
    )
    os.mkdir(path_to_requirements_files)

    with caplog.at_level("INFO"):
        prep._create_requirements_file_for_single_component(
            tmp_dir + "/spec.yaml", path_to_requirements_files
        )

    assert "Found Python package dependencies for component dummy_1" in caplog.text
    with open(path_to_requirements_files + "/dummy_1/requirements.txt", "r") as file:
        lines = file.readlines()
        assert lines == [
            "azureml-core==1.27.0\n",
            "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/\n",
        ]

    pip_dependencies, conda_channels = prep._extract_dependencies_and_channels(
        tmp_dir + "/spec.yaml"
    )
    assert pip_dependencies == [
        "azureml-core==1.27.0",
        "--index-url https://o365exchange.pkgs.visualstudio.com/_packaging/PolymerPythonPackages/pypi/simple/",
    ]
    assert len(conda_channels) == 0

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


def test_create_requirements_file_for_single_component_conda_dependencies_file(caplog):
    clean()
    prep = prepare.Prepare()
    prep.config = Configuration()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.parent.resolve() / "steps/tmp_dir")
    os.mkdir(tmp_dir)
    tmp_yaml = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_2",
        "version": "0.0.1",
        "type": "CommandComponent",
        "command": "pip freeze",
        "code": "../../",
        "environment": {"conda": {"conda_dependencies_file": "conda_env.yaml"}},
    }
    conda_env_yaml = {
        "name": "democomponent_env",
        "channels": [".", "test-channel"],
        "dependencies": [
            "python=3.7",
            {
                "pip": [
                    "azureml-core==1.20.0",
                    "shrike",
                    "numpy==1.19.4",
                ]
            },
        ],
    }

    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)
    with open(tmp_dir + "/conda_env.yaml", "w") as tmp_spec:
        yaml.dump(conda_env_yaml, tmp_spec)

    path_to_requirements_files = (
        prep.config.working_directory + "/tmp_path_to_requirements_files"
    )
    os.mkdir(path_to_requirements_files)

    with caplog.at_level("INFO"):
        prep._create_requirements_file_for_single_component(
            tmp_dir + "/spec.yaml", path_to_requirements_files
        )

    assert "Found Python package dependencies for component dummy_2" in caplog.text
    with open(path_to_requirements_files + "/dummy_2/requirements.txt", "r") as file:
        lines = file.readlines()
        assert lines == ["azureml-core==1.20.0\n", "shrike\n", "numpy==1.19.4\n"]

    pip_dependencies, conda_channels = prep._extract_dependencies_and_channels(
        tmp_dir + "/spec.yaml"
    )
    assert pip_dependencies == ["azureml-core==1.20.0", "shrike", "numpy==1.19.4"]
    assert conda_channels == [".", "test-channel"]

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


def test_create_requirements_file_for_single_component_pip_requirements_file(caplog):
    clean()
    prep = prepare.Prepare()
    prep.config = Configuration()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.parent.resolve() / "steps/tmp_dir")
    os.mkdir(tmp_dir)
    tmp_yaml = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_3",
        "version": "0.0.1",
        "type": "CommandComponent",
        "command": "pip freeze",
        "code": "../../",
        "environment": {"conda": {"pip_requirements_file": "requirements.txt"}},
    }
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)
    with open(tmp_dir + "/requirements.txt", "w") as pip_requirements:
        pip_requirements.writelines(["azureml-defaults\n", "scikit-learn==0.24.2"])

    path_to_requirements_files = (
        prep.config.working_directory + "/tmp_path_to_requirements_files"
    )
    os.mkdir(path_to_requirements_files)

    with caplog.at_level("INFO"):
        prep._create_requirements_file_for_single_component(
            tmp_dir + "/spec.yaml", path_to_requirements_files
        )

    assert "Found Python package dependencies for component dummy_3" in caplog.text
    with open(path_to_requirements_files + "/dummy_3/requirements.txt", "r") as file:
        lines = file.readlines()
        assert lines == ["azureml-defaults\n", "scikit-learn==0.24.2\n"]

    pip_dependencies, conda_channels = prep._extract_dependencies_and_channels(
        tmp_dir + "/spec.yaml"
    )
    assert pip_dependencies == ["azureml-defaults\n", "scikit-learn==0.24.2"]
    assert len(conda_channels) == 0

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


def test_create_requirements_file_for_single_component_no_dependencies(caplog):
    clean()
    prep = prepare.Prepare()
    prep.config = Configuration()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.parent.resolve() / "steps/tmp_dir")
    os.mkdir(tmp_dir)
    tmp_yaml = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_4",
        "version": "0.0.1",
        "type": "CommandComponent",
        "command": "pip freeze",
        "code": "../../",
    }

    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)

    path_to_requirements_files = (
        prep.config.working_directory + "/tmp_path_to_requirements_files"
    )
    os.mkdir(path_to_requirements_files)

    with caplog.at_level("INFO"):
        prep._create_requirements_file_for_single_component(
            tmp_dir + "/spec.yaml", path_to_requirements_files
        )

    assert "Found Python package dependencies for component dummy_4" not in caplog.text
    assert not os.path.exists(path_to_requirements_files + "/dummy_4")

    pip_dependencies, conda_channels = prep._extract_dependencies_and_channels(
        tmp_dir + "/spec.yaml"
    )
    assert len(pip_dependencies) == 0
    assert len(conda_channels) == 0

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_compliance_validation_passed():
    clean()
    prep = prepare.Prepare()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.resolve() / "tmp_dir")
    os.mkdir(tmp_dir)

    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec, open(
        tmp_dir + "/conda_env.yaml", "w"
    ) as tmp_env:
        yaml.dump(SPEC_YAML, tmp_spec)
        yaml.dump(ENV_YAML, tmp_env)

    assert prep.compliance_validation(tmp_dir + "/spec.yaml")

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_compliance_validation_noncompliant_docker_image(caplog):
    clean()
    prep = prepare.Prepare()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.resolve() / "tmp_dir")
    os.mkdir(tmp_dir)

    # non-compliant docker image URL
    tmp_spec_yaml = copy.deepcopy(SPEC_YAML)
    tmp_spec_yaml["environment"]["docker"][
        "image"
    ] = "mcr.microsoft.com/azureml/base-gpu:openmpi3.1.2-cuda10.1-cudnn7-ubuntu18.04"
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec, open(
        tmp_dir + "/conda_env.yaml", "w"
    ) as tmp_env:
        yaml.dump(tmp_spec_yaml, tmp_spec)
        yaml.dump(ENV_YAML, tmp_env)

    component_path = tmp_dir + "/spec.yaml"
    with caplog.at_level("INFO"):
        assert not prep.compliance_validation(component_path)

    assert (
        f"The container base image in {component_path} is not allowed for compliant run."
        in caplog.text
    )

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_compliance_validation_missing_polymerfeed(caplog):
    clean()
    prep = prepare.Prepare()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.resolve() / "tmp_dir")
    os.mkdir(tmp_dir)

    # Polymer Package Feed is missing
    tmp_env_yaml = copy.deepcopy(ENV_YAML)
    tmp_env_yaml["dependencies"][1]["pip"] = ["shrike", "test-package==1.0.0"]
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec, open(
        tmp_dir + "/conda_env.yaml", "w"
    ) as tmp_env:
        yaml.dump(SPEC_YAML, tmp_spec)
        yaml.dump(tmp_env_yaml, tmp_env)

    component_path = tmp_dir + "/spec.yaml"
    with caplog.at_level("INFO"):
        assert not prep.compliance_validation(component_path)

    assert (
        f"The Polymer package feed is not found in environment of {component_path}"
        in caplog.text
    )

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_compliance_validation_noncompliant_feed(caplog):
    clean()
    prep = prepare.Prepare()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.resolve() / "tmp_dir")
    os.mkdir(tmp_dir)

    # Polymer Package Feed is missing
    tmp_env_yaml = copy.deepcopy(ENV_YAML)
    tmp_env_yaml["dependencies"][1]["pip"] = [
        "shrike",
        "test-package==1.0.0",
        "--index-url mcr.microsoft.com/azureml/base-gpu:openmpi3.1.2-cuda10.1-cudnn7-ubuntu18.04",
    ]
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec, open(
        tmp_dir + "/conda_env.yaml", "w"
    ) as tmp_env:
        yaml.dump(SPEC_YAML, tmp_spec)
        yaml.dump(tmp_env_yaml, tmp_env)

    component_path = tmp_dir + "/spec.yaml"
    with caplog.at_level("INFO"):
        assert not prep.compliance_validation(component_path)

    assert (
        f"The package feed in {component_path} is not allowed for compliant run."
        in caplog.text
    )

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_compliance_validation_noncompliant_channels(caplog):
    clean()
    prep = prepare.Prepare()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.resolve() / "tmp_dir")
    os.mkdir(tmp_dir)

    # non-compliant docker image URL
    tmp_env_yaml = copy.deepcopy(ENV_YAML)
    tmp_env_yaml["channels"] = [".", "defaults", "wrong_channel"]
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec, open(
        tmp_dir + "/conda_env.yaml", "w"
    ) as tmp_env:
        yaml.dump(SPEC_YAML, tmp_spec)
        yaml.dump(tmp_env_yaml, tmp_env)

    component_path = tmp_dir + "/spec.yaml"
    with caplog.at_level("INFO"):
        assert not prep.compliance_validation(component_path)

    assert "Only the default conda channel is allowed for compliant run." in caplog.text

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_customized_validation(caplog):
    clean()
    prep = prepare.Prepare()

    # create temporary component for testing
    tmp_dir = str(Path(__file__).parent.resolve() / "tmp_dir")
    os.mkdir(tmp_dir)

    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec, open(
        tmp_dir + "/conda_env.yaml", "w"
    ) as tmp_env:
        yaml.dump(SPEC_YAML, tmp_spec)
        yaml.dump(ENV_YAML, tmp_env)

    assert prep.customized_validation(
        "$.name", "^dum[A-Za-z0-9-_.]+$", tmp_dir + "/spec.yaml"
    )

    assert prep.customized_validation(
        "$.environment.docker.image", "^polymerprod.azurecr.io*", tmp_dir + "/spec.yaml"
    )

    assert prep.customized_validation(
        "$.inputs..description", "^[A-Z].*", tmp_dir + "/spec.yaml"
    )

    with caplog.at_level("INFO"):
        assert not prep.customized_validation(
            "$.name", "^office.smartcompose.[A-Za-z0-9-_.]+$", tmp_dir + "/spec.yaml"
        )

    assert (
        "doesn't match the regular expression ^office.smartcompose.[A-Za-z0-9-_.]+$"
        in caplog.text
    )

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)


@pytest.mark.order(-1)
def test_validate_all_components_enabled_component_validation(caplog, tmp_path):
    clean()
    prep = prepare.Prepare()

    # Create temporary configuration file
    config_path = tmp_path / "aml-build-configuration.yml"
    config_path.write_text(
        f"""component_validation:
  '$.name': '^dum.[A-Za-z0-9-_.]+$'
  '$.environment.docker.image': '^polymerprod.azurecr.io*'
"""
    )
    args = [
        "--configuration-file",
        str(config_path),
        "--enable-component-validation",
    ]

    # Create temporary component folder
    tmp_dir = str(Path(__file__).parent.resolve() / "tmp_dir")
    os.mkdir(tmp_dir)
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec, open(
        tmp_dir + "/conda_env.yaml", "w"
    ) as tmp_env:
        yaml.dump(SPEC_YAML, tmp_spec)
        yaml.dump(ENV_YAML, tmp_env)
    component_path = tmp_dir + "/spec.yaml"

    # Initialize prepare class
    prep = prepare.Prepare()
    prep.config = load_configuration_from_args_and_env(
        args, {"BUILD_SOURCEBRANCH": "refs/heads/main"}
    )
    prep.attach_workspace(TESTING_WORKSPACE)

    with caplog.at_level("INFO"):
        prep.validate_all_components([component_path])

    assert prep._component_statuses[component_path]["validate"] == "succeeded"
    assert not prep._errors
    assert "is valid" in caplog.text

    # Clean up tmp directory
    shutil.rmtree(tmp_dir)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

import argparse
import shrike
from shrike.compliant_logging.constants import DataCategory
import logging

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--prefix", default="SystemLog:")
    parser.add_argument("--log_level", default="INFO")
    args = parser.parse_args()

    shrike.compliant_logging.enable_compliant_logging(
        args.prefix,
        level=args.log_level,
        format="%(prefix)s%(levelname)s:%(name)s:%(message)s",
    )

    # Output will be:
    # WARNING:root:private info
    # SystemLog:WARNING:root:public info
    logging.warning("private info")
    logging.warning("public info", category=DataCategory.PUBLIC)

    logger = logging.getLogger(__name__)

    # Output will be:
    # SystemLog:INFO:__main__:public info
    # SystemLog:WARNING:__main__:public info
    # WARNING:__main__:private info
    logger.info("public info", category=DataCategory.PUBLIC)
    logger.warning("public info", category=DataCategory.PUBLIC)
    logger.warning("private info")



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Federated Learning Example.
"""
from shrike.pipeline import FederatedPipelineBase, StepOutput
import os, sys

SAMPLE_ROOT_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
if SAMPLE_ROOT_PATH not in sys.path:
    print(f"Adding to path: {SAMPLE_ROOT_PATH}")
    sys.path.append(str(SAMPLE_ROOT_PATH))

from subgraph.passthrough_subgraph import PassthroughSubgraph


class SampleFederatedPipeline(FederatedPipelineBase):
    @classmethod
    def required_subgraphs(cls):
        return {"DemoSubgraph": PassthroughSubgraph}

    def preprocess(self, config):
        func = self.component_load("stats_passthrough")
        pipeline_input_dataset = self.dataset_load(
            name=config.democomponent.input_data,
            version=config.democomponent.input_data_version,
        )
        step = func(input_path=pipeline_input_dataset)
        return StepOutput(step, ["output_path"])

    def train(self, config, input, silo):
        func = self.component_load("stats_passthrough")
        step = func(input_path=input[0])
        return StepOutput(step, ["output_path"])

    def midprocess(self, config, input):
        subgraph = self.subgraph_load("DemoSubgraph")
        step = subgraph(input[0])
        return StepOutput(step)

    def postprocess(self, config, input):
        func = self.component_load("stats_passthrough")
        step = func(input_path=input[0])
        return StepOutput(step, ["output_path"])


if __name__ == "__main__":
    SampleFederatedPipeline.main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Hello world example for Spark
"""
# pylint: disable=no-member
# NOTE: because it raises 'dict' has no 'outputs' member in dsl.pipeline construction
import os
import sys
from dataclasses import dataclass
from typing import Optional

from azure.ml.component import dsl
from shrike.pipeline.pipeline_helper import AMLPipelineHelper


class SparkHelloPipeline(AMLPipelineHelper):
    """Runnable/reusable pipeline helper class

    This class inherits from AMLPipelineHelper which provides
    helper functions to create reusable production pipelines for SmartCompose.
    """

    @classmethod
    def get_config_class(cls):
        """Returns the config object (dataclass) for this runnable script.

        Returns:
            dataclass: class for configuring this runnable pipeline.
        """

        @dataclass
        class sparkhelloworld:  # pylint: disable=invalid-name
            """Config object constructed as a dataclass.

            NOTE: the name of this class will be used as namespace in your config yaml file.
            See conf/reference/evaluate_qas_model.yaml for an example.
            """

            # input datasets names (and versions if necessary)
            input_dataset: str = "dummy_dataset"
            input_dataset_version: Optional[str] = "latest"

        # return the dataclass itself
        # for helper class to construct config file
        return sparkhelloworld

    def build(self, config):
        """Builds a pipeline function for this pipeline using AzureML SDK (dsl.pipeline).

        This method should build your graph using the provided config object.
        Your pipeline config will be under config.CONFIGNAME.*
        where CONFIGNAME is the name of the dataclass returned by get_config_class()

        This method returns a constructed pipeline function (decorated with @dsl.pipeline).

        Args:
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            dsl.pipeline: the function to create your pipeline
        """
        # helper function below loads the module from registered or local version
        # depending on your config run.use_local
        spark_hello_component = self.module_load("SparkHelloWorld")

        # Here you should create an instance of a pipeline function (using your custom config dataclass)
        @dsl.pipeline(
            name="Spark Hello World Pipeline (Name)",
            description="Spark Hello World Pipeline (Description)",
            default_datastore=config.compute.compliant_datastore,
        )
        def baseline_pipeline_function(curatedemailreplypairs_dataset):
            """Pipeline function for this graph.

            Args:
                curatedemailreplypairs_dataset (TabularDataset) : the input eyes-on/eyes-off dataset provided as parquet files

            Returns:
                dict[str->PipelineOutputData]: a dictionary of your pipeline outputs
                    for instance to be consumed by other graphs
            """
            # general syntax:
            # module_instance = module_class(input=data, param=value)
            # or
            # subgraph_instance = subgraph_function(input=data, param=value)

            spark_hello_step = spark_hello_component(
                input_path=curatedemailreplypairs_dataset,
                in_file_type="parquet",
                percent_take=42,
                out_file_type="json",
            )
            # each module should be followed by this call
            # in enabled the helper class to fill-up all the settings for this module instance
            self.apply_recommended_runsettings("SparkHelloWorld", spark_hello_step)
            # return {key: output}
            return spark_hello_step.outputs

        # finally return the function itself to be built by helper code
        return baseline_pipeline_function

    def pipeline_instance(self, pipeline_function, config):
        """Given a pipeline function, creates a runnable instance based on provided config.

        This is used only when calling this as a runnable pipeline using .main() function (see below).
        The goal of this function is to map the config to the pipeline_function inputs and params.

        Args:
            pipeline_function (function): the pipeline function obtained from self.build()
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            azureml.core.Pipeline: the instance constructed with its inputs and params.
        """
        # NOTE: self.dataset_load() helps to load the dataset based on its name and version
        pipeline_input_dataset = self.dataset_load(
            name=config.sparkhelloworld.input_dataset,
            version=config.sparkhelloworld.input_dataset_version,
        )

        # when all inputs are obtained, we call the pipeline function
        experiment_pipeline = pipeline_function(pipeline_input_dataset)

        # and we return that function so that helper can run it.
        return experiment_pipeline


# NOTE: main block is necessary only if script is intended to be run from command line
if __name__ == "__main__":
    # calling the helper .main() function
    SparkHelloPipeline.main()



"""
The Azure ML pipeline for running a basic experiment
that consumes a dataset and does some logging.

to execute:
> python pipelines/experiments/demo_count_rows_and_log.py
--config-dir pipelines/config
--config-name experiments/demo_count_rows_and_log
run.submit=True
"""
# pylint: disable=no-member
# NOTE: because it raises 'dict' has no 'outputs' member in dsl.pipeline construction
import os
import sys

from azure.ml.component import dsl
from shrike.pipeline import AMLPipelineHelper

# NOTE: if you need to import from pipelines.*
ACCELERATOR_ROOT_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "..", "..")
)
if ACCELERATOR_ROOT_PATH not in sys.path:
    print(f"Adding to path: {ACCELERATOR_ROOT_PATH}")
    sys.path.append(str(ACCELERATOR_ROOT_PATH))


class CountRowsDemo(AMLPipelineHelper):
    """Runnable/reusable pipeline helper class

    This class inherits from AMLPipelineHelper which provides
    helper functions to create reusable production pipelines.
    """

    def build(self, config):
        """Builds a pipeline function for this pipeline using AzureML SDK (dsl.pipeline).

        This method returns a constructed pipeline function
        (decorated with @dsl.pipeline).

        Args:
            config (DictConfig): configuration object

        Returns:
            dsl.pipeline: the function to create your pipeline
        """

        # helper functions below load the subgraph/component from
        # registered or local version depending on your config.run.use_local
        count_rows_component = self.component_load("CountRowsAndLog")

        # Here you should create an instance of a pipeline function
        # (using your custom config dataclass)
        @dsl.pipeline(
            name="demo-component-count-rows-and-log",
            description="Demo of a component that reads a dataset and counts the rows.",
            default_datastore=config.compute.compliant_datastore,
        )
        def demo_pipeline_function(demo_dataset):
            """Pipeline function for this graph.

            Args:
                demo_dataset: input dataset

            Returns:
                dict[str->PipelineOutputData]: a dictionary of your pipeline outputs
                    for instance to be consumed by other graphs
            """
            # general syntax:
            # component_instance = component_class(input=data, param=value)
            # or
            # subgraph_instance = subgraph_function(input=data, param=value)
            demo_component_step = count_rows_component(input_data=demo_dataset)

            self.apply_recommended_runsettings(
                "CountRowsAndLog", demo_component_step, gpu=False
            )

        # finally return the function itself to be built by helper code
        return demo_pipeline_function

    def pipeline_instance(self, pipeline_function, config):
        """Given a pipeline function, creates a runnable instance based on provided config.

        This is used only when calling this as a runnable pipeline
        using .main() function (see below).
        The goal of this function is to map the config to the
        pipeline_function inputs and params.

        Args:
            pipeline_function (function):
                the pipeline function obtained from self.build()
            config (DictConfig):
                configuration object

        Returns:
            azureml.core.Pipeline: the instance constructed
                                   with its inputs and params.
        """

        # NOTE: self.dataset_load() helps to load the dataset
        # based on its name and version
        pipeline_input_dataset = self.dataset_load(
            name=config.democomponent.input_data,
            version=config.democomponent.input_data_version,
        )

        # we simply call the pipeline function
        demo_pipeline = pipeline_function(demo_dataset=pipeline_input_dataset)

        # and we return that function so that helper can run it.
        return demo_pipeline


# NOTE: main block is necessary only if script is intended to be run from command line
if __name__ == "__main__":
    # calling the helper .main() function
    CountRowsDemo.main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Unit tests for the `register` module.

Consider decorating long end-to-end tests with `@pytest.mark.order(-1)`.
"""

import os
import pytest
import subprocess
import yaml
from random import uniform
from pathlib import Path


from shrike.build.commands import register
from shrike.build.commands import prepare
from shrike.build.core.configuration import (
    Configuration,
    load_configuration_from_args_and_env,
)


TESTING_WORKSPACE = "/subscriptions/48bbc269-ce89-4f6f-9a12-c6f91fcb772d/resourceGroups/github-ci-rg/providers/Microsoft.MachineLearningServices/workspaces/github-ci-ml-wus2"


def test_validate_branch_not_match():
    with pytest.raises(ValueError):
        reg = register.Register()
        reg.config = Configuration(
            source_branch="NOT_MATCH_BRANCH", compliant_branch="^refs/heads/main$"
        )
        reg.validate_branch()


def test_validate_branch(caplog):
    reg = register.Register()
    reg.config = Configuration(
        source_branch="refs/heads/main", compliant_branch="^refs/heads/main$"
    )
    with caplog.at_level("INFO"):
        reg.validate_branch()
    assert "Expected branch: ^refs/heads/main$" in caplog.text
    assert "Current branch: refs/heads/main" in caplog.text
    assert "Current branch matches configured regular expression." in caplog.text


@pytest.mark.order(-1)
@pytest.mark.parametrize("mode", ["foo", "aether", "aml"])
def test_find_signed_component_specification_files(caplog, mode):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    if mode in ["aether", "aml"]:
        # Create catlog files for aether or aml mode
        prep = prepare.Prepare()
        prep.config = Configuration(signing_mode=mode)
        built_component_files = prep.build_all_components(
            prep.find_component_specification_files()
        )
        prep.create_catalog_files(built_component_files)

    reg = register.Register()
    reg.config = Configuration()
    reg.attach_workspace(TESTING_WORKSPACE)

    with caplog.at_level("INFO"):
        reg.find_signed_component_specification_files()

    if mode == "aether":
        assert "Find a signed component for Aether:" in caplog.text
        assert "Cannot find any signed components for AML." in caplog.text
    elif mode == "aml":
        assert "Find 4 signed components for AML." in caplog.text
    else:
        assert "Cannot find any signed components for AML." in caplog.text


@pytest.mark.order(-1)
def test_list_registered_component(caplog):
    reg = register.Register()
    reg.config = Configuration()
    reg.attach_workspace(TESTING_WORKSPACE)

    with caplog.at_level("INFO"):
        reg.list_registered_component()

    assert "DefaultVersion" in caplog.text
    assert not reg._errors

    # Cleanup
    subprocess.run(["git", "clean", "-xdf"])


@pytest.mark.order(-1)
def test_register_all_signed_components_already_exist_component_version(caplog):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    # Create a temporary component folder and generate a random float number as
    # the version number so that we can "almost surely" have a new version number
    # every time we re-run this unit test.
    tmp_dir = str(get_target_path_in_steps_directory("component4/tmp_dir"))
    random_version_number = str(uniform(0.1, 1.0))
    tmp_yaml = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_component_new",
        "version": random_version_number,
        "type": "CommandComponent",
        "command": "pip freeze",
    }
    os.mkdir(tmp_dir)
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)

    # Create catlog files
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="aml", fail_if_version_exists=True)
    built_component_files = prep.build_all_components(
        prep.find_component_specification_files()
    )
    prep.create_catalog_files(built_component_files)

    # Start registration
    reg = register.Register()
    reg.config = Configuration(fail_if_version_exists=True)
    reg.attach_workspace(TESTING_WORKSPACE)

    already_exist_component_path = str(
        get_target_path_in_steps_directory("component4/.build/spec.yaml")
    )
    not_yet_exist_component_path = str(
        get_target_path_in_steps_directory("component4/tmp_dir/.build/spec.yaml")
    )
    with caplog.at_level("INFO"):
        reg.register_all_signed_components(
            [already_exist_component_path, not_yet_exist_component_path]
        )
    assert reg._component_statuses[already_exist_component_path]["register"] == "failed"
    assert (
        reg._component_statuses[not_yet_exist_component_path]["register"] == "succeeded"
    )
    assert len(reg._errors) == 1
    assert (
        "(UserError) Version 0.0.1 already exists in component convert2ss."
        in caplog.text
    )
    assert '"displayName": "dummy_component_new"' in caplog.text
    assert (
        f"{random_version_number} is production-ready. Setting as default."
        in caplog.text
    )

    # Cleanup
    subprocess.run(["git", "clean", "-xdf"])


@pytest.mark.order(-1)
def test_register_all_signed_components_use_build_number(caplog):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    # Create catlog files
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="aml")
    built_component_files = prep.build_all_components(
        prep.find_component_specification_files()
    )
    prep.create_catalog_files(built_component_files)

    # Start registration
    reg = register.Register()
    build_number = os.environ.get("BUILD_BUILDNUMBER") or str(uniform(0.1, 1.0))
    build_sourcebranch = os.environ.get("BUILD_SOURCEBRANCH") or "refs/heads/main"
    reg.config = load_configuration_from_args_and_env(
        args=["--use-build-number"],
        env={
            "BUILD_SOURCEBRANCH": build_sourcebranch,
            "BUILD_BUILDNUMBER": build_number,
        },
    )
    reg.attach_workspace(TESTING_WORKSPACE)

    component_path = str(
        get_target_path_in_steps_directory("component4/.build/spec.yaml")
    )
    with caplog.at_level("INFO"):
        reg.register_all_signed_components([component_path])

    assert reg._component_statuses[component_path]["register"] == "succeeded"
    assert len(reg._errors) == 0

    # Note: There is a trasient error of empty json output generated by az cli.
    # Will uncomment the following line once this error is resolved.
    # assert '"displayName": "Convert Text to StructureStream"' in caplog.text
    assert (
        f"Overwrite the component version with the specified value {build_number}"
        in caplog.text
    )
    assert "is production-ready. Setting as default." in caplog.text
    assert "is not production-ready. NOT setting as default." not in caplog.text

    # Cleanup
    subprocess.run(["git", "clean", "-xdf"])


@pytest.mark.order(-1)
def test_set_default_version_given_prod(caplog):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    # Create a temporary component folder and generate a random float number as
    # the version number so that we can "almost surely" have a new version number
    # every time we re-run this unit test.
    tmp_dir = str(get_target_path_in_steps_directory("tmp_dir"))
    random_version_number = uniform(0.1, 1.0)
    version_numbers = [str(random_version_number) + i for i in ["", ".1", ".1.1"]] + [
        str(int(random_version_number * 100000000))
    ]
    os.mkdir(tmp_dir)
    for idx, version in enumerate(version_numbers):
        tmp_yaml = {
            "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
            "name": "dummy_component_new",
            "version": version,
            "type": "CommandComponent",
            "command": "pip freeze",
        }

        with open(tmp_dir + f"/spec{idx}.yaml", "w") as tmp_spec:
            yaml.dump(tmp_yaml, tmp_spec)

    # Create catlog files
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="aml")
    built_component_files = prep.build_all_components(
        files=[tmp_dir + f"/spec{idx}.yaml" for idx in range(len(version_numbers))]
    )
    prep.create_catalog_files(built_component_files)

    # Start registration
    reg = register.Register()
    reg.config = Configuration()
    reg.attach_workspace(TESTING_WORKSPACE)

    component_path = [
        str(get_target_path_in_steps_directory(f"tmp_dir/.build/spec{idx}.yaml"))
        for idx in range(len(version_numbers))
    ]

    with caplog.at_level("INFO"):
        reg.register_all_signed_components(component_path)
    assert len(reg._errors) == 0
    for idx in range(len(version_numbers)):
        assert reg._component_statuses[component_path[idx]]["register"] == "succeeded"
        assert (
            f"{version_numbers[idx]} is production-ready. Setting as default."
            in caplog.text
        )

    # Cleanup
    subprocess.run(["git", "clean", "-xdf"])


@pytest.mark.order(-1)
def test_set_default_version_given_dev(caplog):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    # Create a temporary component folder and generate a random float number as
    # the version number so that we can "almost surely" have a new version number
    # every time we re-run this unit test.
    tmp_dir = str(get_target_path_in_steps_directory("tmp_dir"))
    random_version_number = uniform(0.1, 1.0)
    version_numbers = [
        str(random_version_number) + i
        for i in ["-dev", "-beta0", "-alpha.1", "-rc1", "alpha"]
    ]
    os.mkdir(tmp_dir)
    for idx, version in enumerate(version_numbers):
        tmp_yaml = {
            "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
            "name": "dummy_component_new",
            "version": version,
            "type": "CommandComponent",
            "command": "pip freeze",
        }

        with open(tmp_dir + f"/spec{idx}.yaml", "w") as tmp_spec:
            yaml.dump(tmp_yaml, tmp_spec)

    # Create catlog files
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="aml")
    built_component_files = prep.build_all_components(
        files=[tmp_dir + f"/spec{idx}.yaml" for idx in range(len(version_numbers))]
    )
    prep.create_catalog_files(built_component_files)

    # Start registration
    reg = register.Register()
    reg.config = Configuration()
    reg.attach_workspace(TESTING_WORKSPACE)

    component_path = [
        str(get_target_path_in_steps_directory(f"tmp_dir/.build/spec{idx}.yaml"))
        for idx in range(len(version_numbers))
    ]

    with caplog.at_level("INFO"):
        reg.register_all_signed_components(component_path)
    assert len(reg._errors) == 0
    for idx in range(len(version_numbers)):
        assert reg._component_statuses[component_path[idx]]["register"] == "succeeded"
        assert (
            f"{version_numbers[idx]} is not production-ready. NOT setting as default."
            in caplog.text
        )

    # Cleanup
    subprocess.run(["git", "clean", "-xdf"])


def test_read_component_version(caplog):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    # Start registration
    reg = register.Register()
    reg.config = Configuration()

    reg.read_component_version("tests/tests_build/steps/component3/spec.yaml")
    reg.read_component_version("tests/tests_build/steps/component2/spec.yaml")
    assert (
        "Component does not have version attribute, attempting to read module version."
        in caplog.text
    )
    assert "Component tests/tests_build/steps/spec.yaml has version 0.0.4."

    # Clean
    subprocess.run(["git", "clean", "-xdf"])


@pytest.mark.order(-1)
def test_register_component_command(caplog):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    # Create a temporary component folder and generate a random float number as
    # the version number so that we can "almost surely" have a new version number
    # every time we re-run this unit test.
    tmp_dir = str(get_target_path_in_steps_directory("tmp_dir"))
    random_version_number = str(uniform(0.1, 1.0))
    tmp_yaml = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_component_new",
        "version": random_version_number,
        "type": "CommandComponent",
        "command": "pip freeze",
    }
    tmp_yaml2 = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_component_new",
        "version": random_version_number + ".0",
        "type": "CommandComponent",
        "command": "pip freeze",
    }
    tmp_yaml3 = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_component_new",
        "version": random_version_number + ".0-beta",
        "type": "CommandComponent",
        "command": "pip freeze",
    }
    os.mkdir(tmp_dir)
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)
    with open(tmp_dir + "/spec2.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml2, tmp_spec)
    with open(tmp_dir + "/spec3.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml3, tmp_spec)

    # Create catlog files
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="aml")
    built_component_files = prep.build_all_components(
        prep.find_component_specification_files()
    )
    prep.create_catalog_files(built_component_files)

    # Start registration
    reg = register.Register()
    reg.config = Configuration()
    reg.attach_workspace(TESTING_WORKSPACE)

    component_path_1 = str(
        get_target_path_in_steps_directory("tmp_dir/.build/spec.yaml")
    )
    component_path_2 = str(
        get_target_path_in_steps_directory("tmp_dir/.build/spec2.yaml")
    )
    component_path_3 = str(
        get_target_path_in_steps_directory("tmp_dir/.build/spec3.yaml")
    )

    with caplog.at_level("INFO"):
        reg.register_all_signed_components(
            [component_path_1, component_path_2, component_path_3]
        )

    assert (
        f"Register command is ml component create --file {component_path_1} --label default\n"
        in caplog.text
    )
    assert (
        f"Register command is ml component create --file {component_path_2} --label default\n"
        in caplog.text
    )
    assert (
        f"Register command is ml component create --file {component_path_3}\n"
        in caplog.text
    )

    build_number = os.environ.get("BUILD_BUILDNUMBER") or str(uniform(0.1, 1.0))
    build_sourcebranch = os.environ.get("BUILD_SOURCEBRANCH") or "refs/heads/main"
    reg.config = load_configuration_from_args_and_env(
        args=["--use-build-number"],
        env={
            "BUILD_SOURCEBRANCH": build_sourcebranch,
            "BUILD_BUILDNUMBER": build_number,
        },
    )
    with caplog.at_level("INFO"):
        reg.register_all_signed_components([component_path_1])
    assert (
        f"Register command is ml component create --file {component_path_1} --version {build_number} --label default\n"
        in caplog.text
    )

    # Clean
    subprocess.run(["git", "clean", "-xdf"])


@pytest.mark.order(-1)
def test_run_with_config_runs_end_to_end(caplog):
    # Clean up .build directories
    subprocess.run(["git", "clean", "-xdf"])

    # Create a temporary component folder and generate a random float number as
    # the version number so that we can "almost surely" have a new version number
    # every time we re-run this unit test.
    tmp_dir = str(get_target_path_in_steps_directory("tmp_dir"))
    random_version_number = str(uniform(0.1, 1.0))
    tmp_yaml = {
        "$schema": "http://azureml/sdk-2-0/CommandComponent.json",
        "name": "dummy_component_new",
        "version": random_version_number,
        "type": "CommandComponent",
        "command": "pip freeze",
    }
    os.mkdir(tmp_dir)
    with open(tmp_dir + "/spec.yaml", "w") as tmp_spec:
        yaml.dump(tmp_yaml, tmp_spec)

    # Create catlog files
    prep = prepare.Prepare()
    prep.config = Configuration(signing_mode="aml")
    built_component_files = prep.build_all_components(files=[tmp_dir + "/spec.yaml"])
    prep.create_catalog_files(built_component_files)

    # Start registration
    reg = register.Register()
    reg.config = Configuration(
        workspaces=[TESTING_WORKSPACE], source_branch="refs/heads/main"
    )
    with caplog.at_level("INFO"):
        reg.run_with_config()

    print(caplog.text)

    assert len(reg._errors) == 0
    assert '"displayName": "dummy_component_new"' in caplog.text
    assert random_version_number in caplog.text
    assert "Find 1 signed components for AML." in caplog.text
    assert "List of components in workspace before current registration." in caplog.text
    assert "List of components in workspace after current registration." in caplog.text

    # Cleanup
    subprocess.run(["git", "clean", "-xdf"])


def get_target_path_in_steps_directory(target) -> Path:
    end_of_path = "steps/" + target
    res = Path(__file__).parent.parent.resolve() / end_of_path
    return res



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
APIs for federated learning.
"""
from shrike import pipeline
from shrike.pipeline.pipeline_config import federated_config, silo
from shrike.pipeline.pipeline_helper import AMLPipelineHelper
from shrike._core import experimental
from coolname import generate_slug
from dataclasses import MISSING, dataclass
from typing import List, Any, Optional, Union
from azure.ml.component import dsl
from azure.ml.component.pipeline import Pipeline
from azure.ml.component.component import Component
import logging
from omegaconf import OmegaConf, open_dict, DictConfig

log = logging.getLogger(__name__)

EXPERIMENTAL_WARNING_MSG = (
    "Federated APIs are experimental and could change at any time."
)


def dataset_name_to_reference(
    component_instance: Component, dataset_names: list
) -> list:
    res = []
    if not dataset_names:
        return res
    for dataset in dataset_names:
        res.append(component_instance.outputs[dataset])
    return res


@dataclass
class StepOutput:
    step: Union[Component, Pipeline] = None
    outputs: List[str] = None


class FederatedPipelineBase(AMLPipelineHelper):
    @experimental(message=EXPERIMENTAL_WARNING_MSG)
    def create_base_name(self) -> str:
        """
        The training outputs from each silo will be stored at <base_name>/<silo_name> on the central storage.
        """
        rv = "fl-"
        rv += generate_slug(2)
        return rv

    @experimental(message=EXPERIMENTAL_WARNING_MSG)
    def preprocess(self, config: DictConfig, input: list) -> StepOutput:
        """
        Define the preprocess step. The outputs will be distributed to each silo's datastore.
        Return: a component/subgraph instance, and a list of output dataset name to be passed to the downstream pipeline **in order**
        """
        pass

    @experimental(message=EXPERIMENTAL_WARNING_MSG)
    def midprocess(self, config: DictConfig, input: list) -> StepOutput:
        """
        Define the midprocess step which reads outputs from `train` in each silo. The outputs will be distributed to each silo's datastore.
        Return: a component/subgraph instance, and a list of output dataset name to be passed to the downstream pipeline **in order**
        """
        pass

    @experimental(message=EXPERIMENTAL_WARNING_MSG)
    def postprocess(self, config: DictConfig, input: list) -> StepOutput:
        """
        Define the postprocess step which reads outputs from `train` in each silo and writes to the `noncompliant_datastore`.
        Return: a component/subgraph instance, and a list of output dataset name to be passed to the downstream pipeline **in order**.
        """
        pass

    @experimental(message=EXPERIMENTAL_WARNING_MSG)
    def train(self, config: DictConfig, input: list, silo: DictConfig) -> StepOutput:
        """
        Define the train step happening at each silo. This reads outputs from `preprocess` or `midprocess`, and sends outputs back to the `noncompliant_datastore`.
        Return: a component/subgraph instance, and a list of output dataset name to be passed to the downstream pipeline **in order**.
        """
        pass

    def data_transfer(
        self, input_list: list, destination_datastore: str, base_name: str
    ) -> list:
        # assume one shared ADF
        dts_name = self.config.federated_config.data_transfer_component
        try:
            data_transfer_component = self.component_load(
                dts_name  # this component is subject to change
            )
            log.info(f"Using {dts_name} as data transfer component.")
        except Exception as e:
            log.error(
                f"Please specify the data transfer component name registered in your workspace in module manifest and `federated_config.data_transfer_component` pipeline yaml."
            )
            raise (e)

        res = []
        if not input_list:
            return res
        for input in input_list:
            data_transfer_step = data_transfer_component(source_data=input)
            self.apply_smart_runsettings(data_transfer_step, datatransfer=True)
            data_transfer_step.outputs.destination_data.configure(
                datastore=destination_datastore,
                path_on_datastore=base_name + "/" + input.port_name,
            )
            res.append(data_transfer_step.outputs.destination_data)
        return res

    def _process_at_orchestrator(self, prev: StepOutput) -> list:
        if isinstance(prev.step, Pipeline):
            prev_output = list(prev.step.outputs.values())
        elif isinstance(prev.step, Component):
            self.apply_smart_runsettings(prev.step)
            prev_output = dataset_name_to_reference(prev.step, prev.outputs)
        else:
            raise Exception(
                f"The output of preprocess/midprocess/postprocess step must be a Pipeline object or a Component object. You are using {type(prev.step)}."
            )
        return prev_output

    def _merge_config(self):
        # override priority: default_config < customized shared config < per-silo setting
        # everything in customized shared config have equal priority
        federated_config = self.config.federated_config
        default_config = federated_config.config_group.get("default_config", {})
        for silo_name, silo_config in federated_config.silos.items():
            customized_config = {}
            if "inherit" in silo_config:
                for inherit_config in silo_config["inherit"]:
                    customized_config = OmegaConf.merge(
                        customized_config, federated_config.config_group[inherit_config]
                    )
            log.info(
                f"=== Original config for silo {silo_name}: {self.config.federated_config.silos[silo_name]}"
            )
            with open_dict(default_config):
                merge = OmegaConf.merge(default_config, customized_config)
            log.info(f"=== Shared config {merge}")
            with open_dict(self.config.federated_config.silos[silo_name]):
                self.config.federated_config.silos[silo_name] = OmegaConf.merge(
                    merge, silo_config
                )
                self.config.federated_config.silos[silo_name].pop("inherit", None)
            log.info(
                f"=== Merged config for silo {silo_name}: {self.config.federated_config.silos[silo_name]}"
            )

    @experimental(message=EXPERIMENTAL_WARNING_MSG)
    def train_in_silo_once(
        self, silo_name: str, silo: DictConfig, prev_output: list, cool_name: str
    ) -> list:
        data_transfer_output = self.data_transfer(
            input_list=prev_output,
            destination_datastore=silo.datastore,
            base_name=cool_name,
        )
        train_step = self.train(self.config, input=data_transfer_output, silo=silo)
        if isinstance(train_step.step, Pipeline):
            train_output = list(train_step.step.outputs.values())
        elif isinstance(train_step.step, Component):
            self.apply_smart_runsettings(
                train_step.step,
                target=silo.compute,
                datastore_name=silo.datastore,
            )
            train_output = dataset_name_to_reference(
                train_step.step, train_step.outputs
            )
        else:
            raise Exception(
                f"The output of train step must be a Pipeline object or a Component object. You are using {type(train_step.step)}"
            )

        silo_output = self.data_transfer(
            input_list=train_output,
            destination_datastore=self.config.compute.noncompliant_datastore,
            base_name=cool_name + "/" + silo_name,
        )
        return silo_output

    @experimental(message=EXPERIMENTAL_WARNING_MSG)
    def build(self, config: DictConfig):
        @dsl.pipeline()
        def pipeline_function():
            self._merge_config()
            prev = self.preprocess(self.config)
            prev_output = self._process_at_orchestrator(prev)

            for iter in range(self.config.federated_config.max_iterations):
                cool_name = self.create_base_name()
                midprocess_input = []
                for silo_name, silo in self.config.federated_config.silos.items():
                    train_output = self.train_in_silo_once(
                        silo_name, silo, prev_output, cool_name
                    )
                    midprocess_input += train_output

                prev = self.midprocess(self.config, input=midprocess_input)
                prev_output = self._process_at_orchestrator(prev)

            prev = self.postprocess(self.config, input=prev_output)
            prev_output = self._process_at_orchestrator(prev)

        return pipeline_function

    def pipeline_instance(self, pipeline_function, config):
        pipeline = pipeline_function()
        return pipeline



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Demonstrate how scrubbing options.
"""

from shrike.compliant_logging.exceptions import prefix_stack_trace

# Output will be:
#
# MyCustomPrefix Traceback (most recent call last):
# MyCustomPrefix   File ".\prefix-stack-trace.py", line 11, in main
# MyCustomPrefix     print(1 / 0)
# MyCustomPrefix ZeroDivisionError: **Exception message scrubbed**
@prefix_stack_trace(prefix="MyCustomPrefix")
def custom_prefix():
    print(1 / 0)


# Output will be:
#
# SystemLog: Traceback (most recent call last):
# SystemLog:   File "/mnt/c/code/shrike/docs/samples/prefix-stack-trace.py", line 20, in scrub2
# SystemLog:     print(1 / 0)
# SystemLog: ZeroDivisionError: Private data was divided by zero
@prefix_stack_trace(scrub_message="Private data was divided by zero")
def custom_message():
    print(1 / 0)


# Output will be:
#
# SystemLog: Traceback (most recent call last):
# SystemLog:   File "/mnt/c/code/shrike/docs/samples/prefix-stack-trace.py", line 24, in scrub3
# SystemLog:     print(1 / 0)
# SystemLog: ZeroDivisionError: division by zero
@prefix_stack_trace(keep_message=True)
def keep_exception_message():
    print(1 / 0)


# Output will be:
#
# SystemLog: Traceback (most recent call last):
# SystemLog:   File "/mnt/c/code/shrike/docs/samples/prefix-stack-trace.py", line 28, in scrub4
# SystemLog:     print(1 / 0)
# SystemLog: ZeroDivisionError: division by zero
@prefix_stack_trace(keep_message=False, allow_list=["ZeroDivision"])
def keep_allowed_exceptions():
    print(1 / 0)


# Output will be:
#
# SystemLog: 2020-11-12 16:56:59 Traceback (most recent call last):
# SystemLog: 2020-11-12 16:56:59   File "prefix-stack-trace.py", line 56, in keep_allowed_exceptions
# SystemLog: 2020-11-12 16:56:59     print(1 / 0)
# SystemLog: 2020-11-12 16:56:59 ZeroDivisionError: **Exception message scrubbed**
@prefix_stack_trace(add_timestamp=True)
def add_timestamp():
    print(1 / 0)


if __name__ == "__main__":
    try:
        custom_prefix()
    except:
        pass
    try:
        custom_message()
    except:
        pass
    try:
        keep_exception_message()
    except:
        pass
    try:
        keep_allowed_exceptions()
    except:
        pass
    try:
        add_timestamp()
    except:
        pass



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

from shrike.build.core.configuration import load_configuration_from_args_and_env
import pytest


def test_load_configuration_from_args_and_env_respects_args():
    args = ["--verbose", "--use-build-number"]
    config = load_configuration_from_args_and_env(
        args, {"BUILD_SOURCEBRANCH": "refs/heads/main", "BUILD_BUILDNUMBER": "1.1.1"}
    )
    assert config.verbose == True
    assert config.use_build_number == True
    assert config.all_component_version == "1.1.1"


def test_load_configuration_all_component_version(caplog):
    args = ["--all-component-version", "2.2.2"]
    with caplog.at_level("INFO"):
        config = load_configuration_from_args_and_env(args, {})
    assert config.all_component_version == "2.2.2"


def test_build_number_overwrite_all_component_version(caplog):
    args = ["--use-build-number", "--all-component-version", "2.2.2"]
    with caplog.at_level("INFO"):
        config = load_configuration_from_args_and_env(
            args,
            {"BUILD_SOURCEBRANCH": "refs/heads/main", "BUILD_BUILDNUMBER": "1.1.1"},
        )
    assert config.use_build_number == True
    assert config.all_component_version == "1.1.1"
    assert (
        "The build number 1.1.1 overwrites the value of all_component_version 2.2.2"
        in caplog.text
    )


def test_both_args_and_file(tmp_path):
    config_path = tmp_path / "aml-build-configuration.yml"
    config_path.write_text("verbose: False")

    args = [
        "--configuration-file",
        str(config_path),
        "--component-specification-glob",
        "*.yaml",
    ]

    config = load_configuration_from_args_and_env(
        args, {"BUILD_SOURCEBRANCH": "refs/heads/main"}
    )

    assert config.configuration_file == str(config_path)
    assert config.component_specification_glob == "*.yaml"
    assert config.verbose == False


def test_configuration_file_alone_works_fine(tmp_path):
    config_path = tmp_path / "aml-build-configuration.yml"
    config_path.write_text("use_build_number: True\nall_component_version: 0.0.0")

    args = ["--configuration-file", str(config_path)]

    # Assert: does not raise.
    config = load_configuration_from_args_and_env(
        args, {"BUILD_SOURCEBRANCH": "refs/heads/main", "BUILD_BUILDNUMBER": "x.x.x"}
    )

    assert config.use_build_number == True
    assert config.all_component_version == "x.x.x"


def test_load_configuration_from_args_and_env_deprecation(caplog, tmp_path):
    config_path = tmp_path / "aml-build-configuration.yml"
    config_path.write_text("fail_if_version_exists: True")
    args1 = ["--allow-duplicate-versions"]
    args2 = ["--allow-duplicate-versions", "--fail-if-version-exists"]
    args3 = ["--allow-duplicate-versions", "--configuration-file", str(config_path)]
    args4 = ["--configuration-file", str(config_path)]

    with pytest.warns(UserWarning) as record:
        config1 = load_configuration_from_args_and_env(args1, {})

    assert config1.fail_if_version_exists == False
    warning_message = [str(i.message) for i in record]
    assert (
        "We recommend against using the parameter allow_duplicate_versions. Please specify fail_if_version_exists instead."
        in warning_message
    )

    with pytest.raises(ValueError):
        config2 = load_configuration_from_args_and_env(args2, {})
        assert config2.fail_if_version_exists

    with pytest.raises(ValueError):
        config3 = load_configuration_from_args_and_env(args3, {})
        assert config3.fail_if_version_exists

    config4 = load_configuration_from_args_and_env(args4, {})
    assert config4.fail_if_version_exists


def test_configuration_path_does_not_exist(capsys, tmp_path):
    config_path = tmp_path / "file_does_not_exist.yaml"
    args = [
        "--configuration-file",
        str(config_path),
        "--component-specification-glob",
        "*.yaml",
    ]
    config = load_configuration_from_args_and_env(args, {})
    captured = capsys.readouterr()

    assert config.component_specification_glob == "*.yaml"
    assert config.configuration_file == str(config_path)
    assert "ERROR: the configuration file path provided" in captured.out


def test_load_configuration_from_env_and_override_behaviour(capsys, tmp_path):
    config_file_path = tmp_path / "aml-build-configuration.yml"
    config_file_path.write_text(
        "fail_if_version_exists: True\nactivation_method: smart\nverbose: False"
    )
    cli_args = [
        "--verbose",
        "--signing-mode",
        "aether",
        "--configuration-file",
        str(config_file_path),
    ]
    env = {"ACTIVATION_METHOD": "all", "signing_mode": "aml"}

    config = load_configuration_from_args_and_env(cli_args, env)
    captured = capsys.readouterr()

    # Testing loading configuration from env is successful
    assert (
        "Merge the config in the environment variables with the config in the command line."
        in captured.out
    )
    assert (
        "Load the config in the environment variables: {'activation_method': 'all', 'signing_mode': 'aml'}"
        in captured.out
    )

    # Testing cli overrides env
    assert config.signing_mode == "aether"

    # Testing env overrides config file
    assert config.activation_method == "all"

    # Testing cli overrides config file
    assert config.verbose is True

    # Testing config file overrides default config
    assert config.fail_if_version_exists is True


def test_configuration_component_validation(tmp_path):
    config_path = tmp_path / "aml-build-configuration.yml"
    config_path.write_text(
        f"""component_validation:
  '$.name': '^office.smartcompose.[A-Za-z0-9-_.]+$'
  '$.environment.docker.image': '^$|^polymerprod.azurecr.io*$'
  '$.inputs..description': '^[A-Z].*'
"""
    )
    args = [
        "--configuration-file",
        str(config_path),
        "--enable-component-validation",
    ]

    config = load_configuration_from_args_and_env(
        args, {"BUILD_SOURCEBRANCH": "refs/heads/main"}
    )

    assert config.configuration_file == str(config_path)
    assert config.enable_component_validation
    assert (
        config.component_validation["$.name"] == "^office.smartcompose.[A-Za-z0-9-_.]+$"
    )
    assert (
        config.component_validation["$.environment.docker.image"]
        == "^$|^polymerprod.azurecr.io*$"
    )
    assert config.component_validation["$.inputs..description"] == "^[A-Z].*"



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

"""
Constructor for a test pipeline for SmartCompose.
"""
import os
import sys
import argparse

from azure.ml.component import dsl
from azureml.core import Dataset
from shrike.pipeline.pipeline_helper import AMLPipelineHelper

from dataclasses import dataclass
from typing import Dict
from omegaconf import MISSING


class BuidTimeConditionalPassthroughSubgraph(AMLPipelineHelper):
    """sample pipeline builder class"""

    @classmethod
    def get_config_class(cls):
        @dataclass
        class passthrough:  # pylint: disable=invalid-name
            input_dataset: str = "dummy_dataset"

        return passthrough

    def build(self, config):
        """Builds a pipeline function for this pipeline.

        Args:
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            pipeline_function: the function to create your pipeline
        """
        # this function just loads modules either locally or remotely depending on use_local list
        stats_passthrough = self.module_load("stats_passthrough")
        stats_passthrough_windows = self.module_load("stats_passthrough_windows")

        # create an instance of a pipeline function using parameters
        @dsl.pipeline(
            name="TutorialPipeline",
            description="Just runs the Probe module in AML",
            default_datastore=config.compute.compliant_datastore,
        )
        def tutorial_pipeline_function(input_dataset):
            """Constructs a sequence of steps to ...

            Args:
                None

            Returns:
                dict: output_path
            """
            stats_step = stats_passthrough(input_path=input_dataset)
            self.apply_recommended_runsettings("stats_passthrough", stats_step)

            if config.subgraph.passthrough_windows:
                stats_step_2 = stats_passthrough_windows(
                    input_path=stats_step.outputs.output_path
                )
                self.apply_recommended_runsettings(
                    "stats_passthrough_windows", stats_step_2
                )

                return {"output_path": stats_step_2.outputs.output_path}
            else:
                return {"output_path": stats_step.outputs.output_path}

        return tutorial_pipeline_function

    def pipeline_instance(self, pipeline_function, config):
        """Creates an instance of the pipeline using arguments.

        Args:
            pipeline_function (function): the pipeline function obtained from self.build()
            config (DictConfig): configuration object (see get_config_class())

        Returns:
            pipeline: the instance constructed using build() function
        """
        pipeline_input_dataset = self.dataset_load(config.passthrough.input_dataset)

        runnable_pipeline = pipeline_function(pipeline_input_dataset)
        return runnable_pipeline


if __name__ == "__main__":
    BuidTimeConditionalPassthroughSubgraph.main()



"""
The AML subgraph for demo graph
"""
# pylint: disable=no-member
# NOTE: because it raises 'dict' has no 'outputs' member in dsl.pipeline construction
import os
import sys

from azure.ml.component import dsl
from shrike.pipeline.pipeline_helper import AMLPipelineHelper

# NOTE: if you need to import from pipelines.*
ACCELERATOR_ROOT_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "..", "..")
)
if ACCELERATOR_ROOT_PATH not in sys.path:
    print(f"Adding to path: {ACCELERATOR_ROOT_PATH}")
    sys.path.append(str(ACCELERATOR_ROOT_PATH))


class DemoSubgraph(AMLPipelineHelper):
    """Runnable/reusable pipeline helper class

    This class inherits from AMLPipelineHelper which provides
    helper functions to create reusable production pipelines.
    """

    def build(self, config):
        component = self.component_load("AggregateModelWeights")

        @dsl.pipeline(
            name="demo-subgraph",
            description="The AML pipeline for the demo subgraph",
            default_datastore=config.compute.compliant_datastore,
        )
        def demosubgraph_pipeline_function(
            input_data_01,
            input_data_02,
            input_data_03,
        ):
            component_step_1 = component(
                input_data_01=input_data_01,
                input_data_02=input_data_02,
            )

            self.apply_recommended_runsettings(
                "AggregateModelWeights", component_step_1, gpu=True
            )

            component_step_2 = component(
                input_data_01=component_step_1.outputs.results,
                input_data_02=input_data_03,
            )

            self.apply_recommended_runsettings(
                "AggregateModelWeights", component_step_2, gpu=True
            )

            # return {key: output}
            return {"subgraph_results": component_step_2.outputs.results}

        # finally return the function itself to be built by helper code
        return demosubgraph_pipeline_function
